package assays.com;

import java.util.HashMap;
import java.util.List;
import java.util.Iterator;
import assays.hibernate.AssayPersistent;
import assays.hibernate.StudyGroup;
import assays.hibernate.AssayUser;
import assays.hibernate.UserGroup;
import assays.hibernate.MetadataManager;

public final class FileString {
  private String oldContents;
  private String newContents;
  private String userFileName;
  private String userFilePath;
  private String userTypeId;
  private String userTypeName;
  private String kindOfFile;
  private String newSuffix = null;
  private HashMap<Long,String> typeBook = null;
  private boolean noContent = false;
  private int level;
  private boolean isData;
  private Long sysId;

//structure of filestring
// \rsuffix\rfilename\rtypid\r.....

  public FileString( int level, boolean isData, String cont, Long sysId){
    oldContents = cont;
    newContents = cont;
    userFileName = "";
    userFilePath = "";
    userTypeId = "0";
    userTypeName = "";
    kindOfFile = "";
    this.level = level;
    this.isData = isData;
    this.sysId = sysId;
  }

  public void clearFileString(){
    oldContents = "";
    newContents = "";
    userFileName = "";
    userFilePath = "";
    userTypeId = "0";
    userTypeName = "";
    kindOfFile = "";
    noContent = true;
  }

  public Long getUserTypeId(){
	if (userTypeId.equals("0")) return new Long(-1); //select first
    return new Long(userTypeId);
  }

  public String getUserFilePath(){
    return userFilePath;
  }

  public String getKindOfFile(){
    return kindOfFile;
  }

  public void setTypeBook( List types){
    typeBook = new HashMap<Long,String>();
    typeBook.put(new Long(0),"");
    if (types!=null){
      Iterator typesIter = types.iterator();
      while (typesIter.hasNext()) {
        AssayPersistent type = (AssayPersistent) typesIter.next();
        typeBook.put(type.getId(),type.getVisibleName());
      }
    }
  }

  private String getFreeSuffix(){
    int i = 1, j, k = 0, l;
    if (oldContents==null || oldContents.length() < 5 ||
        oldContents.charAt(0)!='\r') return "1";
    while (i < oldContents.length()){
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = oldContents.substring(i,j);
      l = Util.getIntegerValue(suffix);
      if (l > k) k = l;
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
    }
    return String.valueOf( k + 1);
  }

  private static int getFileCount( String contents){
    int i = 1, j, k = 0;
    if (contents==null || contents.length() < 5 ||
   		contents.charAt(0)!='\r') return 0;
    while (i < contents.length()){
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    return k;
  }  
// used by delete all
  public String[] getAllFileNames(){
    int k = getFileCount( oldContents);
    if (k==0) return null;
    String[] names = new String[k];
    int i = 1, j; k = 0; 
    while (i < oldContents.length()){
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = oldContents.substring(i,j);
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      names[k] = getFileName(suffix);
      i = j + 1;
      j = oldContents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    return names;
  }

// used by delete deleted
  public String[] getDeletedFileNames(){
    int kOld = getFileCount( oldContents);
    if (kOld==0) return null;
    int iOld, iNew, j, k, kNew = 0;
    String newSuf;
    iNew = 1; newSuf = null;
    while (iNew < newContents.length()){
      j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
        j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
        j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
        kNew++;
      }
      if (kOld <= kNew) return null;
      String[] names = new String[kOld - kNew];
      iNew = 1;
      if (iNew < newContents.length()){
        j = newContents.indexOf('\r', iNew);
        newSuf = newContents.substring(iNew,j);
        iNew = j + 1;
        j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
        j = newContents.indexOf('\r', iNew);
        iNew = j + 1;
      } else newSuf = null;
      k = 0; iOld = 1;
      while (iOld < oldContents.length()){
        j = oldContents.indexOf('\r', iOld);
        String suffix = oldContents.substring(iOld,j);
        iOld = j + 1;
        if (!suffix.equals(newSuf)){
          names[k] = getFileName(suffix);
//          names[k] = prefix+"_"+sysId.toString()+"_"+suffix;
          k++;
        }
        else {
          if (iNew < newContents.length()) {
            j = newContents.indexOf('\r', iNew);
            newSuf = newContents.substring(iNew, j);
            iNew = j + 1;
            j = newContents.indexOf('\r', iNew);
            iNew = j + 1;
            j = newContents.indexOf('\r', iNew);
            iNew = j + 1;
          }
          else newSuf = null;
        }
        j = oldContents.indexOf('\r', iOld);
        iOld = j + 1;
        j = oldContents.indexOf('\r', iOld);
        iOld = j + 1;
      }
      return names;
    }

  public String getHTML( StudyGroup study, boolean isFT){ //used by edit pages
    AssayUser owner = null;
    UserGroup ownerGroup = null;
    if (study!=null){
      owner = study.getOwner();
      ownerGroup = study.getOwnerGroup();
    }
    String ownerStr1 = (owner==null)?"0":owner.getId().toString();
    String ownerStr2 = (ownerGroup==null)?"0":ownerGroup.getId().toString();
    String prefix = getPrefix( level, isData);
    String changeType = MetadataManager.getString("changeType");
    StringBuffer xyz = new StringBuffer();
    int i = 1; int j, k = 0;
    if (newContents!=null && newContents.length() > 0 && newContents.charAt(0)=='\r')
    while (i < newContents.length()){
      j = newContents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = newContents.substring(i,j);
      i = j + 1;
      j = newContents.indexOf('\r', i);
      if (j < 0) break;
      String fileName = newContents.substring(i,j);
      i = j + 1;
      j = newContents.indexOf('\r', i);
      if (j < 0) break;
      String typeId = newContents.substring(i,j);

      String key = prefix + String.valueOf(++k);
      xyz.append("<span id =\"s")
         .append(key)
         .append("\">");
//visible part - file reference
      xyz.append("<a href=\"javascript:checkFB('")
         .append(getFileName(suffix))
         .append("','")
         .append(fileName)
         .append("','")
         .append(ownerStr1)
         .append("','")
         .append(ownerStr2)
         .append("');\">");
      xyz.append(fileName)
         .append("</a>&nbsp;");
    if (isFT){
//visible part - file type name
      xyz.append("of type:&nbsp;<a id=\"u")
         .append(key)
         .append("\">")
         .append((String)typeBook.get( Util.getLong( typeId)))
         .append("</a>&nbsp;");
//visible part - change type button
      xyz.append("<input type=\"submit\" value=\"")
      	 .append(changeType) 	
         .append("\" onclick='return change")
         .append((isData)?"DFType(\"":"PFType(\"")
         .append(key)
         .append("\")'/>");
    }
//visible part - delete button
     xyz.append("<input type=\"submit\" value=\"Remove\"")
        .append(" onclick='return deleteFile(\"s")
        .append(key)
        .append("\")'/>");
//unvisible part - user file name
      xyz.append("<input type=\"hidden\" name =\"")
         .append(key)
         .append("\" value=\"")
         .append(fileName)
         .append("\">");
//unvisible part - file name suffix
      xyz.append("<input type=\"hidden\" name =\"h")
         .append(key)
         .append("\" value=\"")
         .append(suffix)
         .append("\">");
//unvisible part - file type id
     xyz.append("<input type=\"hidden\" name =\"t")
        .append(key)
        .append("\" id=\"t")
        .append(key)
        .append("\" value=\"")
        .append(typeId)
        .append("\"><br>");
      xyz.append("</span>\r\n");
      i = j + 1;
    }
//unvisible part - number of files   "lprefix"
    xyz.append("<input type=\"hidden\" name =\"l")
       .append(prefix)
       .append("\" value=\"")
       .append(String.valueOf(k))
       .append("\"/>");
//unvisible part - local or ftp file? "fprefix"
       xyz.append("<input type=\"hidden\" name =\"f")
          .append(prefix)
          .append("\" id=\"f")
          .append(prefix)
          .append("\" value=\"")    //"u" or "f"
          .append(kindOfFile)
          .append("\"/>");
//unvisible part - full user file name  "vprefix"
       xyz.append("<input type=\"hidden\" name =\"v")
          .append(prefix)
          .append("\" id=\"v")
          .append(prefix)
          .append("\" value=\"")
          .append(userFilePath)
          .append("\"/>");
//unvisible part - file type id
       xyz.append("<input type=\"hidden\" name =\"tt")
          .append(prefix)
          .append("\" id=\"tt")
          .append(prefix)
          .append("\" value=\"")
          .append(userTypeId)
          .append("\"/>");
//unvisible part - file type name
       xyz.append("<input type=\"hidden\" name =\"tn")
       .append(prefix)
       .append("\" id=\"tn")
       .append(prefix)
       .append("\" value=\"")
       .append(userTypeName)
       .append("\"/>");
//visible part - user file name  'prefix"
    xyz.append("<input type=\"text\" size=\"50\" name =\"")
       .append(prefix)
       .append("\" id=\"")
       .append(prefix)
       .append("\" readonly style=\"background-color:#E5EAF0\"")
       .append(" value=\"")
       .append(userFileName)
       .append("\"/>");
    if (isFT){
//  visible part - file type name
    xyz.append("&nbsp;of type:&nbsp;<a id=\"u")
       .append(prefix)
    .append("\">")
    .append(userTypeName)
    .append("</a>&nbsp;");
//visible part - change type button
 xyz.append("<input type=\"submit\" value=\"")
   	.append(changeType) 	
 	.append("\" onclick='return change")
    .append((isData)?"DF":"PF")
    .append("Type0()'/>");
    } 
//visible part - clear button
xyz.append("<input type=\"submit\" value=\"Clear\"")
   .append(" onclick='return clear")
   .append((isData)?"DF":"PF")
   .append("File()'/>");
    return xyz.toString();
  }

  private String getFileName( String suffix){
    String prefix = getPrefix( level, isData);
    String ident = (sysId!=null)?sysId.toString():"";
    String extension = (isData)?".dat":".pro";
    return prefix+"_"+ident+"_"+suffix+extension;
  }

  public String getFileName( Long sysId){
    String prefix = getPrefix( level, isData);
    String extension = (isData)?".dat":".pro";
    return (userFilePath!=null && !userFilePath.equals(""))?
          (prefix+"_"+sysId.toString()+"_"+newSuffix+extension):null;
  }
//after user dialog creates string to be saved
//and prepares info for methods called by UserHttpSession
//-getKindOfFile
//-getFileName
//-getUserFilePath
  public String getContents( RequestParser request){
    String prefix = getPrefix( level, isData);
    int k, l;
    if (noContent){ noContent = false; return "";}
    StringBuffer xyz = new StringBuffer();
    String suffix, fileName, typeId;
    String count = request.getParameter("l" + prefix);
    k = 0; l = (count!=null)?Util.getIntegerValue(count):10;
    while (k++ < l){
      String key = prefix + String.valueOf(k);
      suffix = request.getParameter("h" + key);
      if (suffix != null) {  //not removed
        fileName = request.getParameter(key);
        typeId = request.getParameter("t" + key);
        xyz.append('\r')
           .append(suffix)
           .append('\r')
           .append(fileName)
           .append('\r')
           .append(typeId);
      }
    }
    xyz.append('\r');
    newContents = xyz.toString();

    userFilePath = request.getParameter("v"+prefix);
    userFileName = request.getParameter(prefix);
    userTypeId = request.getParameter("tt"+prefix);
    userTypeName = request.getParameter("tn"+prefix);
//    Util.writeLog("getContents", userTypeName, userTypeId);
    kindOfFile = request.getParameter("f"+prefix);
// Append to contents new file entered
    if (userFilePath!=null && !userFilePath.equals("")){
       newSuffix = getFreeSuffix();
       xyz.append(newSuffix)
          .append('\r')
          .append(userFileName)
          .append('\r')
          .append(userTypeId)
          .append('\r');
    }
    return xyz.toString();
  }
//------------------------- static methods -------------------------
  private static String getPrefix( int level, boolean isData){
    switch (level){
      case 0: return isData?"viewd":"viewp";
      case 1: return isData?"stgrd":"stgrp";
      default: return isData?"assayd":"assayp";
    }
  }

  private static String getNewFileName( int level, boolean isData, Long sysId){
      String prefix = getPrefix( level, isData);
      String extention = (isData)?".dat":".pro";
      return prefix + '_' + sysId.toString() + "_1" + extention;
  }

  public static String getNewFileName( int level, boolean isData,
                                       Long sysId, int suffix){
      String prefix = getPrefix( level, isData);
      String extention = (isData)?".dat":".pro";
      return prefix + '_' + sysId.toString() + '_' +
                                String.valueOf(suffix) + extention;
  }

  private static String getNewFileName( int level, boolean isData,
                                       Long sysId, String suffix){
      String prefix = getPrefix( level, isData);
      String extention = (isData)?".dat":".pro";
      return prefix + '_' + sysId.toString() + '_' +
                                suffix + extention;
  }

  private static String getNewContents( String fileName, String typeId){
      return "\r1\r" + fileName + '\r' + typeId + '\r';
  }

  public static String getNewContents( String fileName, String typeId, int suffix){
      return String.valueOf(suffix) + '\r' + fileName + '\r' + typeId + '\r';
  }
  
  public static String getFileListDF( int level,
    String source, Long sysId, StudyGroup study, boolean isPending, int colnum){
    String suffix, userfilename;
	int k = getFileCount( source);
	if (k==0) return null;
    StringBuffer xyz = new StringBuffer();
    if (isPending){
      xyz.append("<img border=\"0\" title=\"");
      xyz.append(getFileList(source));
      xyz.append("\" src=\"./images/hourglass_a.gif\">");
    } else {
      String sysIds = sysId.toString();	
      AssayUser owner = (study==null)?null:study.getOwner();
      UserGroup ownerGroup = (study==null)?null:study.getOwnerGroup();
      String ownerStr1 = (owner==null)?"0":owner.getId().toString();
      String ownerStr2 = (ownerGroup==null)?"0":ownerGroup.getId().toString();
      int i = 1; int j;
      if (k==1){   	
        j = source.indexOf('\r', i);
        suffix = source.substring(i, j); 
        i = j + 1;
        j = source.indexOf('\r', i);
        userfilename = source.substring(i, j);    	
        xyz.append("<a href=\"javascript:checkFB('")
        .append(getPrefix( level, true))  //
        .append('_')
        .append(sysIds)
        .append('_')
        .append(suffix)
        .append(".dat','")   //
        .append(userfilename)
        .append("','")
        .append(ownerStr1)
        .append("','")
        .append(ownerStr2)
        .append("');\">");   
        xyz.append("<img border=\"0\" title=\"")
        .append(userfilename)
        .append("\" src=\"./images/datafiles_a.gif\"></a>");  //
      } else {
        int k1 = 0; 	  
        StringBuffer titleBuf = new StringBuffer();
	    while (i < source.length()) {
		  j = source.indexOf('\r', i);
		  if (j < 0)break;
		  suffix = source.substring(i, j); 
		  i = j + 1;
		  j = source.indexOf('\r', i);
		  if (j < 0)break;
		  userfilename = source.substring(i, j);
          i = j + 1;
		  j = source.indexOf('\r', i);
		  if (j < 0) break;
		  i = j + 1;
          if (k1 > 0) titleBuf.append(", ");
          titleBuf.append( userfilename);
          xyz.append("<input name=\"")
        	.append(userfilename)
        	.append("\" id=\"")
        	.append(sysIds)
        	.append("D")       //
        	.append(k1)
        	.append("\" type=\"hidden\" value=\"");
          xyz.append("javascript:checkFB('")
          .append(getPrefix( level, true))    //
          .append('_')
          .append(sysIds)
          .append('_')
          .append(suffix)
          .append(".dat','")      //
          .append(userfilename)
          .append("','")
          .append(ownerStr1)
          .append("','")
          .append(ownerStr2)
          .append("')");   
          xyz.append("\"/>");
          k1++;
	    }
        xyz.append("<a href=\"javascript:actionShowDF('")   //
        .append(sysIds)
        .append("','")
        .append(colnum)
        .append("')\">");
        xyz.append("<img border=\"0\" id=\"D-")     //
        .append(sysIds)
        .append("\" title=\"");
        xyz.append(titleBuf.toString());
        xyz.append("\" src=\"./images/datafiles_down_a.gif\"></a>");  //
      }
    }
    return xyz.toString();
  } 
  public static String getFileListPF( int level,
    String source, Long sysId, StudyGroup study, boolean isPending, int colnum){
    String suffix, userfilename;
	int k = getFileCount( source);
	if (k==0) return null;
    StringBuffer xyz = new StringBuffer();
    if (isPending){
      xyz.append("<img border=\"0\" title=\"");
      xyz.append(getFileList(source));
      xyz.append("\" src=\"./images/hourglass_a.gif\">");
    } else {
      String sysIds = sysId.toString();	
      AssayUser owner = (study==null)?null:study.getOwner();
      UserGroup ownerGroup = (study==null)?null:study.getOwnerGroup();
      String ownerStr1 = (owner==null)?"0":owner.getId().toString();
      String ownerStr2 = (ownerGroup==null)?"0":ownerGroup.getId().toString();
      int i = 1; int j;
      if (k==1){   	
        j = source.indexOf('\r', i);
        suffix = source.substring(i, j); 
        i = j + 1;
        j = source.indexOf('\r', i);
        userfilename = source.substring(i, j);    	
        xyz.append("<a href=\"javascript:checkFB('")
        .append(getPrefix( level, false))  //
        .append('_')
        .append(sysIds)
        .append('_')
        .append(suffix)
        .append(".pro','")     //
        .append(userfilename)
        .append("','")
        .append(ownerStr1)
        .append("','")
        .append(ownerStr2)
        .append("');\">");   
        xyz.append("<img border=\"0\" title=\"")
        .append(userfilename)
        .append("\" src=\"./images/protofiles_a.gif\"></a>");  //
      } else {
        int k1 = 0; 	  
        StringBuffer titleBuf = new StringBuffer();
	    while (i < source.length()) {
		  j = source.indexOf('\r', i);
		  if (j < 0)break;
		  suffix = source.substring(i, j); 
		  i = j + 1;
		  j = source.indexOf('\r', i);
		  if (j < 0)break;
		  userfilename = source.substring(i, j);
          i = j + 1;
		  j = source.indexOf('\r', i);
		  if (j < 0) break;
		  i = j + 1;
          if (k1 > 0) titleBuf.append(", ");
          titleBuf.append( userfilename);
          xyz.append("<input name=\"")
        	.append(userfilename)
        	.append("\" id=\"")
        	.append(sysIds)
        	.append("P")     //
        	.append(k1)
        	.append("\" type=\"hidden\" value=\"");
          xyz.append("javascript:checkFB('")
          .append(getPrefix( level, false))     //
          .append('_')
          .append(sysIds)
          .append('_')
          .append(suffix)
          .append(".pro','")    //
          .append(userfilename)
          .append("','")
          .append(ownerStr1)
          .append("','")
          .append(ownerStr2)
          .append("')");   
          xyz.append("\"/>");
          k1++;
	    }
        xyz.append("<a href=\"javascript:actionShowPF('")  //
        .append(sysIds)
        .append("','")
        .append(colnum)
        .append("')\">");
        xyz.append("<img border=\"0\" id=\"P-")  //
        .append(sysIds)
        .append("\" title=\"");
        xyz.append(titleBuf.toString());
        xyz.append("\" src=\"./images/protofiles_down_a.gif\"></a>");  //
      }
    }
    return xyz.toString();
  } 
  public static String getFileList( String source){
    if (source!=null && source.length() > 5 && source.charAt(0)=='\r'){
      StringBuffer xyz = new StringBuffer();
      int i = 1; int k = 0; int j;
      while (i < source.length()) {
        j = source.indexOf('\r', i);
        if (j < 0)break;
        i = j + 1;
        j = source.indexOf('\r', i);
        if (j < 0)break;
        if (k++ > 0) xyz.append(',');
        xyz.append(source.substring(i, j));
        i = j + 1;
        j = source.indexOf('\r', i);
        if (j < 0)break;
        i = j + 1;
      }
      return xyz.toString();
    }
    return null;
  }

//used by delete study group with assays
  public static String[] getAllFileNames( int level, boolean isData,
      String contents, Long sysId){
    int i, j;
    int k = getFileCount(contents);
    if (k==0) return null;
    String[] names = new String[k];
    k = 0; i = 1;
//2)get file names
    while (i < contents.length()){
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = contents.substring(i,j);
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      names[k] = getNewFileName( level, isData, sysId, suffix);
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    return names;
  }

//used by Martins_EGA  getAllFileNamesII( 1, true, contents, sysId);
  public static String[] getAllFileNamesII( int level, boolean isData,
      String contents, Long sysId){
    int i, j;
    int k = getFileCount(contents);
    if (k==0) return null;
    String[] names = new String[k];
    k = 0; i = 1;
//2)get file names
    String preffix = getPrefix( level, isData); 
    String extention = (isData)?".dat":".pro";
    String ident = sysId.toString();
    while (i < contents.length()){
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      String suffix = contents.substring(i,j);
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      names[k] = preffix + '_' + ident + "_" + suffix + extention;
      i = j + 1;
      j = contents.indexOf('\r', i);
      if (j < 0) break;
      i = j + 1;
      k++;
    }
    return names;
  }
  
  public static String getExportStatusImage( Integer exportStatus, String transferName){
    String rezult = "";
    if (exportStatus!= null){
      int exportStatusValue = exportStatus.intValue();
      if (exportStatusValue > 0){
    	rezult = "<img border=\"0\" title=\""+
    	         transferName+" transfer\" src=\"./images/export_\r.gif\">";
        switch (exportStatusValue){
          case 1000: rezult = rezult.replace('\r','p');break;
          case 2000: rezult = rezult.replace('\r','s');break;
          case 3000: rezult = rezult.replace('\r','f');break;
          case 4000: rezult = rezult.replace('\r','f');break;
          case 5000: rezult = rezult.replace('\r','c');break;
        }
      }	
    }
    return rezult;  	
  }  
}
