package assays.com;

import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.util.Stack;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.util.List;
import java.util.Iterator;

import javax.servlet.jsp.JspWriter;

import assays.hibernate.*;
import assays.services.*;
/**
 *
 * <p>Title: </p>
 *
 * <p>Description:Exports technology metadata in XML format
 *    used by ExportTechnology.jsp called from AdminPage.jsp
 *  </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public final class XMLExport {
  private UserHttpSession session;
  private Stack<String> tagStack;
  private boolean isStarted;
  private String prevName;
  private StringBuffer myOut;

  private JspWriter jspOut;
  private String encoding;

  public XMLExport( UserHttpSession tmpHttpSessObj, JspWriter out, String enc) {
    session = tmpHttpSessObj;
    jspOut = out;
    encoding = enc;
    tagStack = new Stack<String>();
  }

  private void startTag( String name) throws IOException {
    if (isStarted){
      jspOut.print('>');
      tagStack.push(prevName);
    }
    jspOut.println();
    jspOut.print('<');
    jspOut.print(name);
    prevName = name;
    isStarted = true;
  }

  private void endTag() throws IOException {
    if (isStarted){
      jspOut.print("/>");
      isStarted = false;
    } else {
      jspOut.println();
      prevName = (String)tagStack.pop();
      jspOut.print("</");
      jspOut.print(prevName);
      jspOut.print('>');
    }
  }

  private void attribute(String name, String value) throws IOException {
    jspOut.print(' ');
    jspOut.print(name);
    jspOut.print("=\"");
    jspOut.print(Util.getHTMLString(value));
    jspOut.print('\"');
  }
  private void header()throws IOException{
    jspOut.print("<?xml version=\"1.0\" encoding=\"");
    jspOut.print(encoding);
    jspOut.print("\"?>");
  }

  public void exportTechn(Technology techn) throws Exception {
	header();
	startTag(XMLTags.XAIMS_Export);
	attribute(XMLTags.Xversion,"1.17");

	  startTag(XMLTags.XTransferTypes);
	  List plist = session.getAllObjects(TransferType.class);
	  if (plist != null) {
	    Iterator piter = plist.iterator();
	    while (piter.hasNext()) {
	  	TransferType obj = (TransferType) piter.next();
	      startTag(XMLTags.XTransferType);
	      attribute(XMLTags.Xname, obj.getVisibleName());
	      attribute(XMLTags.Xabbr, obj.getAbbr());
	      attribute(XMLTags.Xcomment, obj.getComment());
	      attribute(XMLTags.XdataDirectory, obj.getDataDirectory());
	      attribute(XMLTags.XtransferLevel, obj.getTransferFromLevel().toString());
	      endTag(); //TransferType
	    }
	  }
	  endTag(); //TransferTypes
	  
	  startTag(XMLTags.XParameterGroups);
	  plist = session.getAllObjects(ParameterGroup.class);
	  if (plist != null) {
	    Iterator piter = plist.iterator();
	    while (piter.hasNext()) {
	      ParameterGroup obj = (ParameterGroup) piter.next();
	      startTag(XMLTags.XGroup);
	      attribute(XMLTags.Xname, obj.getVisibleName());
	      attribute(XMLTags.Xshortname, obj.getShortName());
	      attribute(XMLTags.Xcomment, Util.getHTMLString(obj.getComment()));
	        startTag(XMLTags.XAssayConf);
	        attribute(XMLTags.Xrow_numb, obj.getAssayPosition().toString());
	        endTag();
	        startTag(XMLTags.XSGConf);
	        attribute(XMLTags.Xrow_numb, obj.getStgrPosition().toString());
	        endTag();
	        startTag(XMLTags.XViewConf);
	        attribute(XMLTags.Xrow_numb, obj.getViewPosition().toString());
	        endTag();
	      endTag(); //Group
	    }
	  }
	  endTag(); //ParameterGroups
	  startTag(XMLTags.XCommonVocabularies);
	  plist = session.getAllObjectsForTechnology(Vocabulary.class, null);
	  if (plist != null) {
	    Iterator piter = plist.iterator();
	    while (piter.hasNext()) {
	      Vocabulary obj = (Vocabulary) piter.next();
	      startTag(XMLTags.XVocabulary);
	      attribute(XMLTags.Xtable, obj.getVisibleName());
	      attribute(XMLTags.Xname, obj.getLongName());
	      attribute(XMLTags.Xcomment, obj.getComment());
	      List vlist = obj.getValueList();
	      List klist = obj.getKeyList();
	      Iterator kiter = klist.iterator();
	      Iterator viter = vlist.iterator();
	      while (kiter.hasNext()) {
	        startTag(XMLTags.XItem);
	        attribute(XMLTags.Xname, (String) viter.next());
	        attribute(XMLTags.Xseq_numb, (String) kiter.next());
	        endTag();//item
	      }
	      endTag(); //common vocabulary
	    }
	  }
	  endTag(); //common vocabularies
	  startTag(XMLTags.XCommonParameterTypes);
	  plist = session.getAllObjectsForTechnology(ParameterType.class, null);
	  if (plist != null) {
	    Iterator piter = plist.iterator();
	    while (piter.hasNext()) {
	      ParameterType obj = (ParameterType) piter.next();
	      startTag(XMLTags.XParameterType);
	      attribute(XMLTags.Xname, obj.getVisibleName());
	      attribute(XMLTags.Xbasetype, AssayTypeName.getVisibleName(obj.getBaseType()));
	      if (obj.getDisplay1()!=null)
	        attribute(XMLTags.Xattr1, obj.getDisplay1().toString());
	      if (obj.getDisplay2()!=null)
	        attribute(XMLTags.Xattr2, obj.getDisplay2().toString());
	      if (obj.getVocabulary()!=null)
	        attribute(XMLTags.Xtable, obj.getVocabulary().getVisibleName());
	      attribute(XMLTags.Xpopup, obj.getPopupName());
	      attribute(XMLTags.Xcomment, obj.getComment());
	      endTag(); //parameter type
	    }
	  }
	  endTag(); //Common parameter types
	  startTag(XMLTags.XDefaultCommonParameters);
	  plist = session.getAllObjectsForTechnology(CommonParameter.class,
	      null);
	  if (plist != null) {
	    Iterator piter = plist.iterator();
	    while (piter.hasNext()) {
	      CommonParameter obj = (CommonParameter) piter.next();
	      startTag(XMLTags.XParameter);
	      attribute(XMLTags.Xname, obj.getLongName());
	      attribute(XMLTags.Xupload, obj.getVisualUploadName());
	      attribute(XMLTags.Xshortname, obj.getVisibleName());
	      attribute(XMLTags.Xfilter, obj.isShowInFilter() ? "1" : "0");
	      attribute(XMLTags.Xcomment, obj.getComment());
	      String uvz = obj.getAssayColumnName();
	      if (uvz != null && !uvz.equals("")) {
	        startTag(XMLTags.XAssayConf);
	        attribute(XMLTags.Xcolumn, uvz);
	        attribute(XMLTags.Xcol_numb, obj.getAssayViewColumnNumber().toString());
	        attribute(XMLTags.Xsortable, obj.isAssaySortable() ? "1" : "0");
	        endTag();
	      }
	      uvz = obj.getStgrColumnName();
	      if (uvz != null && !uvz.equals("")) {
	        startTag(XMLTags.XSGConf);
	        attribute(XMLTags.Xcolumn, uvz);
	        attribute(XMLTags.Xcol_numb, obj.getStgrViewColumnNumber().toString());
	        attribute(XMLTags.Xsortable, obj.isStgrSortable() ? "1" : "0");
	        endTag();
	      }
	      uvz = obj.getViewColumnName();
	      if (uvz != null && !uvz.equals("")) {
	        startTag(XMLTags.XViewConf);
	        attribute(XMLTags.Xcolumn, uvz);
	        attribute(XMLTags.Xcol_numb, obj.getViewViewColumnNumber().toString());
	        attribute(XMLTags.Xsortable, obj.isViewSortable() ? "1" : "0");
	        endTag();
	      }
	      endTag(); //Parameter
	    }
	  }
	  endTag(); //Default Common parameters

	  if (techn!=null){
	    startTag(XMLTags.XTechnology);
	    attribute(XMLTags.Xname, techn.getVisibleName());
	    attribute(XMLTags.Xprefix, techn.getIdPrefix());
	    attribute(XMLTags.Xftp, techn.getVisibleFtpDirectory());
	    attribute(XMLTags.Xpublic, techn.isPublic() ? "1" : "0");
	    TransferType tt = techn.getTransferType();
	    if (tt!=null)
	      attribute(XMLTags.XTransferType, tt.getVisibleName());
	    Integer svl = techn.getShowViewLevelsX();
	    attribute(XMLTags.XviewLevel, (svl != null)?svl.toString():"0");
	    attribute(XMLTags.XuseIdGener, techn.useIdGeneration()? "1" : "0");
	    attribute(XMLTags.Xcomment, techn.getComment());

	    startTag(XMLTags.XVocabularies);
	    plist = session.getAllObjectsForTechnology(Vocabulary.class, techn);
	    if (plist != null) {
	      Iterator piter = plist.iterator();
	      while (piter.hasNext()) {
	        Vocabulary obj = (Vocabulary) piter.next();
	        startTag(XMLTags.XVocabulary);
	        attribute(XMLTags.Xtable, obj.getVisibleName());
	        attribute(XMLTags.Xname, obj.getLongName());
	        attribute(XMLTags.Xcomment, obj.getComment());
	        List vlist = obj.getValueList();
	        List klist = obj.getKeyList();
	        Iterator kiter = klist.iterator();
	        Iterator viter = vlist.iterator();
	        while (kiter.hasNext()) {
	          startTag(XMLTags.XItem);
	          attribute(XMLTags.Xname, (String) viter.next());
	          attribute(XMLTags.Xseq_numb, (String) kiter.next());
	          endTag();//item
	        }
	        endTag(); //vocabulary
	      }
	    }
	    endTag(); //vocabularies
	    startTag(XMLTags.XParameterTypes);
	    plist = session.getAllObjectsForTechnology(ParameterType.class, techn);
	    if (plist != null) {
	      Iterator piter = plist.iterator();
	      while (piter.hasNext()) {
	        ParameterType obj = (ParameterType) piter.next();
	        startTag(XMLTags.XParameterType);
	        attribute(XMLTags.Xname, obj.getVisibleName());
	        attribute(XMLTags.Xbasetype, AssayTypeName.getVisibleName(obj.getBaseType()));
	        if (obj.getDisplay1()!=null)
	          attribute(XMLTags.Xattr1, obj.getDisplay1().toString());
	        if (obj.getDisplay2()!=null)
	          attribute(XMLTags.Xattr2, obj.getDisplay2().toString());
	        if (obj.getVocabulary()!=null)
	          attribute(XMLTags.Xtable, obj.getVocabulary().getVisibleName());
	        attribute(XMLTags.Xpopup, obj.getPopupName());
	        attribute(XMLTags.Xcomment, obj.getComment());
	        endTag(); //parameter type
	      }
	    }
	    endTag(); //parameter types

	    startTag(XMLTags.XCommonParameters);
	    plist = session.getAllObjectsForTechnology(CommonParameter.class,
	        techn);
	    if (plist != null) {
	      Iterator piter = plist.iterator();
	      while (piter.hasNext()) {
	        CommonParameter obj = (CommonParameter) piter.next();
	        startTag(XMLTags.XParameter);
	        attribute(XMLTags.Xname, obj.getLongName());
	        attribute(XMLTags.Xupload, obj.getVisualUploadName());
	        attribute(XMLTags.Xshortname, obj.getVisibleName());
	        attribute(XMLTags.Xfilter, obj.isShowInFilter() ? "1" : "0");
	        attribute(XMLTags.Xcomment, obj.getComment());
	        String uvz = obj.getAssayColumnName();
	        if (uvz != null && !uvz.equals("")) {
	          startTag(XMLTags.XAssayConf);
	          attribute(XMLTags.Xcolumn, uvz);
	          attribute(XMLTags.Xcol_numb, obj.getAssayViewColumnNumber().toString());
	          attribute(XMLTags.Xsortable, obj.isAssaySortable() ? "1" : "0");
	          endTag();
	        }
	        uvz = obj.getStgrColumnName();
	        if (uvz != null && !uvz.equals("")) {
	          startTag(XMLTags.XSGConf);
	          attribute(XMLTags.Xcolumn, uvz);
	          attribute(XMLTags.Xcol_numb, obj.getStgrViewColumnNumber().toString());
	          attribute(XMLTags.Xsortable, obj.isStgrSortable() ? "1" : "0");
	          endTag();
	        }
	        uvz = obj.getViewColumnName();
	        if (uvz != null && !uvz.equals("")) {
	          startTag(XMLTags.XViewConf);
	          attribute(XMLTags.Xcolumn, uvz);
	          attribute(XMLTags.Xcol_numb, obj.getViewViewColumnNumber().toString());
	          attribute(XMLTags.Xsortable, obj.isViewSortable() ? "1" : "0");
	          endTag();
	        }
	        endTag(); //Parameter
	      }
	    }
	    endTag(); //Common parameters
	    startTag(XMLTags.XParameters);
	    plist = session.getAllObjectsForTechnology(Parameter.class, techn);
	    if (plist != null) {
	      Iterator piter = plist.iterator();
	      while (piter.hasNext()) {
	        Parameter obj = (Parameter) piter.next();
	        startTag(XMLTags.XParameter);
	        attribute(XMLTags.Xname, obj.getLongName());
	        attribute(XMLTags.Xupload, obj.getVisualUploadName());
	        attribute(XMLTags.Xshortname, obj.getVisibleName());
	        attribute(XMLTags.Xgroup, obj.getParameterGroup().getVisibleName());
	        attribute(XMLTags.Xtype, obj.getParameterType().getVisibleName());
	        attribute(XMLTags.Xfilter, obj.isShowInFilter() ? "1" : "0");
	        attribute(XMLTags.Xrequired, obj.isRequired() ? "1" : "0");
	        attribute(XMLTags.Xnote, obj.getNoteToDisplay());
	        attribute(XMLTags.Xcomment, obj.getComment());
	        String uvz = obj.getAssayColumnName();
	        if (uvz != null && !uvz.equals("")) {
	          startTag(XMLTags.XAssayConf);
	          attribute(XMLTags.Xcolumn, uvz);
	          attribute(XMLTags.Xcol_numb, obj.getAssayViewColumnNumber().toString());
	          attribute(XMLTags.Xsortable, obj.isAssaySortable() ? "1" : "0");
	          attribute(XMLTags.Xrow_numb, obj.getAssayRowNumber().toString());
	          endTag();
	        }
	        uvz = obj.getStgrColumnName();
	        if (uvz != null && !uvz.equals("")) {
	          startTag(XMLTags.XSGConf);
	          attribute(XMLTags.Xcolumn, uvz);
	          attribute(XMLTags.Xcol_numb, obj.getStgrViewColumnNumber().toString());
	          attribute(XMLTags.Xsortable, obj.isStgrSortable() ? "1" : "0");
	          attribute(XMLTags.Xrow_numb, obj.getStgrRowNumber().toString());
	          endTag();
	        }
	        uvz = obj.getViewColumnName();
	        if (uvz != null && !uvz.equals("")) {
	          startTag(XMLTags.XViewConf);
	          attribute(XMLTags.Xcolumn, uvz);
	          attribute(XMLTags.Xcol_numb, obj.getViewViewColumnNumber().toString());
	          attribute(XMLTags.Xsortable, obj.isViewSortable() ? "1" : "0");
	          attribute(XMLTags.Xrow_numb, obj.getViewRowNumber().toString());
	          endTag();
	        }
	        endTag(); //Parameter
	      }
	    }
	    endTag(); //Parameters
	    startTag(XMLTags.XDataFileTypes);
	    plist = session.getAllObjectsForTechnology(DataType.class, techn);
	    if (plist != null) {
	      Iterator piter = plist.iterator();
	      while (piter.hasNext()) {
	        DataType obj = (DataType) piter.next();
	        startTag(XMLTags.XFileType);
	        attribute(XMLTags.Xtype, obj.getVisibleName());
	        attribute(XMLTags.Xfile_ext, obj.getFileExtension());
	        attribute(XMLTags.Xcomment, obj.getComment());
	        endTag();
	      }
	    }
	    endTag(); //data file types
	    startTag(XMLTags.XProtocolFileTypes);
	    plist = session.getAllObjectsForTechnology(ProtocolType.class, techn);
	    if (plist != null) {
	      Iterator piter = plist.iterator();
	      while (piter.hasNext()) {
	        ProtocolType obj = (ProtocolType) piter.next();
	        startTag(XMLTags.XFileType);
	        attribute(XMLTags.Xtype, obj.getVisibleName());
	        attribute(XMLTags.Xfile_ext, obj.getFileExtension());
	        attribute(XMLTags.Xcomment, obj.getComment());
	        endTag();
	      }
	    }
      endTag(); //protocol file types
      endTag(); //technology
    }
    endTag();//AIMS_Export
  }
//----------------------------------------------------------------------  
//----------------------------------------------------------------------  
  private void create_xml_for_list( List t_list, int level) throws Exception{
      List plist = t_list;
      if (plist != null) {
    	int num = 0;   
        DecimalFormat snf = (DecimalFormat)NumberFormat.getNumberInstance();
        snf.applyPattern("'F'000");
        Iterator piter = plist.iterator();
        while (piter.hasNext()) {
          TechnologyParameter obj = (TechnologyParameter) piter.next();
          startTag(XMLTags.XParameter);
          attribute(XMLTags.Xname, obj.param_name_long);
          attribute(XMLTags.Xupload, obj.param_name_upload);
          attribute(XMLTags.Xshortname, obj.param_name_short);
          ParameterGroup pgr = (ParameterGroup)session.getObjectById(ParameterGroup.class,
        		  obj.param_group);
          attribute(XMLTags.Xgroup, pgr.getVisibleName());
          ParameterType ptp = (ParameterType)session.getObjectById(ParameterType.class,
        		  obj.param_type);
          attribute(XMLTags.Xtype, ptp.getVisibleName());
          attribute(XMLTags.Xfilter, obj.show_in_filter.toString());
          attribute(XMLTags.Xrequired, obj.is_required.toString());
          attribute(XMLTags.Xnote, obj.note_to_display);
          String colName = snf.format(++num);
          if (level == 2){
            startTag(XMLTags.XAssayConf);
            attribute(XMLTags.Xcolumn, colName);
            endTag();
          } else
          if (level == 1){
            startTag(XMLTags.XSGConf);
            attribute(XMLTags.Xcolumn, colName);
            endTag();
          } else
          if (level == 0){
            startTag(XMLTags.XViewConf);
            attribute(XMLTags.Xcolumn, colName);
            endTag();
          }
          endTag(); //Parameter
        }
      }
  }	  
  public void create_xml_for_technology( List t_list, List m_list, List b_list) throws Exception{
    startTag(XMLTags.XAIMS_Export);
    attribute(XMLTags.Xversion,"1.17");

    startTag(XMLTags.XDefaultCommonParameters);
    List plist = session.getAllObjectsForTechnology(CommonParameter.class,
        null);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        CommonParameter obj = (CommonParameter) piter.next();
        startTag(XMLTags.XParameter);
        attribute(XMLTags.Xname, obj.getLongName());
        attribute(XMLTags.Xupload, obj.getVisualUploadName());
        attribute(XMLTags.Xshortname, obj.getVisibleName());
        attribute(XMLTags.Xfilter, obj.isShowInFilter() ? "1" : "0");
        attribute(XMLTags.Xcomment, obj.getComment());
        String uvz = obj.getAssayColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag(XMLTags.XAssayConf);
          attribute(XMLTags.Xcolumn, uvz);
          attribute(XMLTags.Xcol_numb, obj.getAssayViewColumnNumber().toString());
          attribute(XMLTags.Xsortable, obj.isAssaySortable() ? "1" : "0");
          endTag();
        }
        uvz = obj.getStgrColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag(XMLTags.XSGConf);
          attribute(XMLTags.Xcolumn, uvz);
          attribute(XMLTags.Xcol_numb, obj.getStgrViewColumnNumber().toString());
          attribute(XMLTags.Xsortable, obj.isStgrSortable() ? "1" : "0");
          endTag();
        }
        uvz = obj.getViewColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag(XMLTags.XViewConf);
          attribute(XMLTags.Xcolumn, uvz);
          attribute(XMLTags.Xcol_numb, obj.getViewViewColumnNumber().toString());
          attribute(XMLTags.Xsortable, obj.isViewSortable() ? "1" : "0");
          endTag();
        }

        endTag(); //Default Common parameter
      }
    }
    endTag(); //Default Common parameters
    startTag(XMLTags.XParameters);
    create_xml_for_list( t_list, 0);
    create_xml_for_list( m_list, 1);
    create_xml_for_list( b_list, 2);
    endTag(); //Parameters
    endTag(); //AIMS_Export
  }
//----------------------------------------------------------------------
//----------------------------------------------------------------------
  public XMLExport( UserHttpSession tmpHttpSessObj) {
    session = tmpHttpSessObj;
    myOut = new StringBuffer();
    tagStack = new Stack<String>();
    isStarted = false;
  }

  private void startTag2( String name) throws IOException {
    if (isStarted){
      myOut.append('>');
      tagStack.push(prevName);
    }
    myOut.append("\r\n");
    myOut.append('<');
    myOut.append(name);
    prevName = name;
    isStarted = true;
  }
  private void endTag2() throws IOException {
    if (isStarted){
      myOut.append("/>");
      isStarted = false;
    } else {
      myOut.append("\r\n");
      prevName = (String)tagStack.pop();
      myOut.append("</");
      myOut.append(prevName);
      myOut.append('>');
    }
  }
  private void attribute2(String name, String value) throws IOException {
	myOut.append(' ');
	myOut.append(name);
	myOut.append("=\"");
	myOut.append(Util.getHTMLString(value));
	myOut.append('\"');
  }
  private void create_xml_for_list2( List t_list, int level) throws Exception{
    List plist = t_list;
    if (plist != null) {
      int num = 0;   
      DecimalFormat snf = (DecimalFormat)NumberFormat.getNumberInstance();
      snf.applyPattern("'F'000");
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        TechnologyParameter obj = (TechnologyParameter) piter.next();
        startTag2(XMLTags.XParameter);
        attribute2(XMLTags.Xname, obj.param_name_long);
        attribute2(XMLTags.Xupload, obj.param_name_upload);
        attribute2(XMLTags.Xshortname, obj.param_name_short);
        ParameterGroup pgr = (ParameterGroup)session.getObjectById(ParameterGroup.class,
        		  obj.param_group);
        attribute2(XMLTags.Xgroup, pgr.getVisibleName());
        ParameterType ptp = (ParameterType)session.getObjectById(ParameterType.class,
        		  obj.param_type);
        attribute2(XMLTags.Xtype, ptp.getVisibleName());
        attribute2(XMLTags.Xfilter, obj.show_in_filter.toString());
        attribute2(XMLTags.Xrequired, obj.is_required.toString());
        attribute2(XMLTags.Xnote, obj.note_to_display);
        String colName = snf.format(++num);
        if (level == 2){
          startTag2(XMLTags.XAssayConf);
          attribute2(XMLTags.Xcolumn, colName);
          endTag2();
        } else
        if (level == 1){
          startTag2(XMLTags.XSGConf);
          attribute2(XMLTags.Xcolumn, colName);
          endTag2();
        } else
        if (level == 0){
          startTag2(XMLTags.XViewConf);
          attribute2(XMLTags.Xcolumn, colName);
          endTag2();
        }
        endTag2(); //Parameter
      }
    }
  }	  
  public String create_xml_for_technology2( List t_list, List m_list, List b_list) throws Exception{
    startTag2(XMLTags.XAIMS_Export);
    attribute2(XMLTags.Xversion,"1.17");

    startTag2(XMLTags.XDefaultCommonParameters);
    List plist = session.getAllObjectsForTechnology(CommonParameter.class,
        null);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        CommonParameter obj = (CommonParameter) piter.next();
        startTag2(XMLTags.XParameter);
        attribute2(XMLTags.Xname, obj.getLongName());
        attribute2(XMLTags.Xupload, obj.getVisualUploadName());
        attribute2(XMLTags.Xshortname, obj.getVisibleName());
        attribute2(XMLTags.Xfilter, obj.isShowInFilter() ? "1" : "0");
        attribute2(XMLTags.Xcomment, obj.getComment());
        String uvz = obj.getAssayColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag2(XMLTags.XAssayConf);
          attribute2(XMLTags.Xcolumn, uvz);
          attribute2(XMLTags.Xcol_numb, obj.getAssayViewColumnNumber().toString());
          attribute2(XMLTags.Xsortable, obj.isAssaySortable() ? "1" : "0");
          endTag2();
        }
        uvz = obj.getStgrColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag2(XMLTags.XSGConf);
          attribute2(XMLTags.Xcolumn, uvz);
          attribute2(XMLTags.Xcol_numb, obj.getStgrViewColumnNumber().toString());
          attribute2(XMLTags.Xsortable, obj.isStgrSortable() ? "1" : "0");
          endTag2();
        }
        uvz = obj.getViewColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag2(XMLTags.XViewConf);
          attribute2(XMLTags.Xcolumn, uvz);
          attribute2(XMLTags.Xcol_numb, obj.getViewViewColumnNumber().toString());
          attribute2(XMLTags.Xsortable, obj.isViewSortable() ? "1" : "0");
          endTag2();
        }
        endTag2(); //Default Common parameter
      }
    }
    endTag2(); //Default Common parameters
    startTag2(XMLTags.XParameters);
    create_xml_for_list2( t_list, 0);
    create_xml_for_list2( m_list, 1);
    create_xml_for_list2( b_list, 2);
    endTag2(); //Parameters
    endTag2(); //AIMS_Export
    return myOut.toString();
  }

}
