package assays.com;

import javax.servlet.jsp.JspWriter;
import java.util.Stack;
import assays.hibernate.*;
import java.io.IOException;
import java.util.List;
import java.util.Iterator;

import org.hibernate.StatelessSession;
import org.hibernate.Query;
import org.hibernate.ScrollableResults;
import org.hibernate.ScrollMode;
import java.lang.reflect.Method;
/**
 *
 * <p>Title: </p>
 *
 * <p>Description:Exports technology metadata and data in XML format
 *    used by ExportTechnologyData.jsp called from AdminPage.jsp.
 *    Pagaidam netiek lietots
 *  </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public final class XMLExportTechnologyData {

private UserHttpSession session;
private Stack<String> tagStack;
private boolean isStarted = false;
private JspWriter jspOut;
private String prevName;
private String encoding;

public XMLExportTechnologyData( UserHttpSession tmpHttpSessObj, JspWriter out, String enc) {
  session = tmpHttpSessObj;
  jspOut = out;
  encoding = enc;
  tagStack = new Stack<String>();
}

private void startTag( String name) throws IOException {
  if (isStarted){
    jspOut.print('>');
    tagStack.push(prevName);
  }
  jspOut.println();
  jspOut.print('<');
  jspOut.print(name);
  prevName = name;
  isStarted = true;
}

private void endTag() throws IOException {
  if (isStarted){
    jspOut.print("/>");
    isStarted = false;
  } else {
    jspOut.println();
    prevName = (String)tagStack.pop();
    jspOut.print("</");
    jspOut.print(prevName);
    jspOut.print('>');
  }
}

private void attribute(String name, String value) throws IOException {
  jspOut.print(' ');
  jspOut.print(name);
  jspOut.print("=\"");
  jspOut.print(Util.getHTMLString(value));
  jspOut.print('\"');
}
private void header()throws IOException{
  jspOut.print("<?xml version=\"1.0\" encoding=\"");
  jspOut.print(encoding);
  jspOut.print("\"?>");
}

public void exportTechn(Technology techn) throws Exception {
  header();
  startTag(XMLTags.XAIMS_Export);
  attribute(XMLTags.Xversion,"1.17");

  startTag(XMLTags.XTransferTypes);
  List plist = session.getAllObjects(TransferType.class);
  if (plist != null) {
    Iterator piter = plist.iterator();
    while (piter.hasNext()) {
  	TransferType obj = (TransferType) piter.next();
      startTag(XMLTags.XTransferType);
      attribute(XMLTags.Xname, obj.getVisibleName());
      attribute(XMLTags.Xabbr, obj.getAbbr());
      attribute(XMLTags.Xcomment, obj.getComment());
      attribute(XMLTags.XdataDirectory, obj.getDataDirectory());
      attribute(XMLTags.XtransferLevel, obj.getTransferFromLevel().toString());
      endTag(); //TransferType
    }
  }
  endTag(); //TransferTypes
  
  startTag(XMLTags.XParameterGroups);
  plist = session.getAllObjects(ParameterGroup.class);
  if (plist != null) {
    Iterator piter = plist.iterator();
    while (piter.hasNext()) {
      ParameterGroup obj = (ParameterGroup) piter.next();
      startTag(XMLTags.XGroup);
      attribute(XMLTags.Xname, obj.getVisibleName());
      attribute(XMLTags.Xshortname, obj.getShortName());
      attribute(XMLTags.Xcomment, Util.getHTMLString(obj.getComment()));
        startTag(XMLTags.XAssayConf);
        attribute(XMLTags.Xrow_numb, obj.getAssayPosition().toString());
        endTag();
        startTag(XMLTags.XSGConf);
        attribute(XMLTags.Xrow_numb, obj.getStgrPosition().toString());
        endTag();
        startTag(XMLTags.XViewConf);
        attribute(XMLTags.Xrow_numb, obj.getViewPosition().toString());
        endTag();
      endTag(); //Group
    }
  }
  endTag(); //ParameterGroups
  startTag(XMLTags.XCommonVocabularies);
  plist = session.getAllObjectsForTechnology(Vocabulary.class, null);
  if (plist != null) {
    Iterator piter = plist.iterator();
    while (piter.hasNext()) {
      Vocabulary obj = (Vocabulary) piter.next();
      startTag(XMLTags.XVocabulary);
      attribute(XMLTags.Xtable, obj.getVisibleName());
      attribute(XMLTags.Xname, obj.getLongName());
      attribute(XMLTags.Xcomment, obj.getComment());
      List vlist = obj.getValueList();
      List klist = obj.getKeyList();
      Iterator kiter = klist.iterator();
      Iterator viter = vlist.iterator();
      while (kiter.hasNext()) {
        startTag(XMLTags.XItem);
        attribute(XMLTags.Xname, (String) viter.next());
        attribute(XMLTags.Xseq_numb, (String) kiter.next());
        endTag();//item
      }
      endTag(); //common vocabulary
    }
  }
  endTag(); //common vocabularies
  startTag(XMLTags.XCommonParameterTypes);
  plist = session.getAllObjectsForTechnology(ParameterType.class, null);
  if (plist != null) {
    Iterator piter = plist.iterator();
    while (piter.hasNext()) {
      ParameterType obj = (ParameterType) piter.next();
      startTag(XMLTags.XParameterType);
      attribute(XMLTags.Xname, obj.getVisibleName());
      attribute(XMLTags.Xbasetype, AssayTypeName.getVisibleName(obj.getBaseType()));
      if (obj.getDisplay1()!=null)
        attribute(XMLTags.Xattr1, obj.getDisplay1().toString());
      if (obj.getDisplay2()!=null)
        attribute(XMLTags.Xattr2, obj.getDisplay2().toString());
      if (obj.getVocabulary()!=null)
        attribute(XMLTags.Xtable, obj.getVocabulary().getVisibleName());
      attribute(XMLTags.Xpopup, obj.getPopupName());
      attribute(XMLTags.Xcomment, obj.getComment());
      endTag(); //parameter type
    }
  }
  endTag(); //Common parameter types
  startTag(XMLTags.XDefaultCommonParameters);
  plist = session.getAllObjectsForTechnology(CommonParameter.class,
      null);
  if (plist != null) {
    Iterator piter = plist.iterator();
    while (piter.hasNext()) {
      CommonParameter obj = (CommonParameter) piter.next();
      startTag(XMLTags.XParameter);
      attribute(XMLTags.Xname, obj.getLongName());
      attribute(XMLTags.Xupload, obj.getVisualUploadName());
      attribute(XMLTags.Xshortname, obj.getVisibleName());
      attribute(XMLTags.Xfilter, obj.isShowInFilter() ? "1" : "0");
      attribute(XMLTags.Xcomment, obj.getComment());
      String uvz = obj.getAssayColumnName();
      if (uvz != null && !uvz.equals("")) {
        startTag(XMLTags.XAssayConf);
        attribute(XMLTags.Xcolumn, uvz);
        attribute(XMLTags.Xcol_numb, obj.getAssayViewColumnNumber().toString());
        attribute(XMLTags.Xsortable, obj.isAssaySortable() ? "1" : "0");
        endTag();
      }
      uvz = obj.getStgrColumnName();
      if (uvz != null && !uvz.equals("")) {
        startTag(XMLTags.XSGConf);
        attribute(XMLTags.Xcolumn, uvz);
        attribute(XMLTags.Xcol_numb, obj.getStgrViewColumnNumber().toString());
        attribute(XMLTags.Xsortable, obj.isStgrSortable() ? "1" : "0");
        endTag();
      }
      uvz = obj.getViewColumnName();
      if (uvz != null && !uvz.equals("")) {
        startTag(XMLTags.XViewConf);
        attribute(XMLTags.Xcolumn, uvz);
        attribute(XMLTags.Xcol_numb, obj.getViewViewColumnNumber().toString());
        attribute(XMLTags.Xsortable, obj.isViewSortable() ? "1" : "0");
        endTag();
      }
      endTag(); //Parameter
    }
  }
  endTag(); //Default Common parameters

  if (techn!=null){
    startTag(XMLTags.XTechnology);
    attribute(XMLTags.Xname, techn.getVisibleName());
    attribute(XMLTags.Xprefix, techn.getIdPrefix());
    attribute(XMLTags.Xftp, techn.getVisibleFtpDirectory());
    attribute(XMLTags.Xpublic, techn.isPublic() ? "1" : "0");
    TransferType tt = techn.getTransferType();
    if (tt!=null)
      attribute(XMLTags.XTransferType, tt.getVisibleName());
    Integer svl = techn.getShowViewLevelsX();
    attribute(XMLTags.XviewLevel, (svl != null)?svl.toString():"0");
    attribute(XMLTags.XuseIdGener, techn.useIdGeneration()? "1" : "0");
    attribute(XMLTags.Xcomment, techn.getComment());

    startTag(XMLTags.XVocabularies);
    plist = session.getAllObjectsForTechnology(Vocabulary.class, techn);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        Vocabulary obj = (Vocabulary) piter.next();
        startTag(XMLTags.XVocabulary);
        attribute(XMLTags.Xtable, obj.getVisibleName());
        attribute(XMLTags.Xname, obj.getLongName());
        attribute(XMLTags.Xcomment, obj.getComment());
        List vlist = obj.getValueList();
        List klist = obj.getKeyList();
        Iterator kiter = klist.iterator();
        Iterator viter = vlist.iterator();
        while (kiter.hasNext()) {
          startTag(XMLTags.XItem);
          attribute(XMLTags.Xname, (String) viter.next());
          attribute(XMLTags.Xseq_numb, (String) kiter.next());
          endTag();//item
        }
        endTag(); //vocabulary
      }
    }
    endTag(); //vocabularies
    startTag(XMLTags.XParameterTypes);
    plist = session.getAllObjectsForTechnology(ParameterType.class, techn);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        ParameterType obj = (ParameterType) piter.next();
        startTag(XMLTags.XParameterType);
        attribute(XMLTags.Xname, obj.getVisibleName());
        attribute(XMLTags.Xbasetype, AssayTypeName.getVisibleName(obj.getBaseType()));
        if (obj.getDisplay1()!=null)
          attribute(XMLTags.Xattr1, obj.getDisplay1().toString());
        if (obj.getDisplay2()!=null)
          attribute(XMLTags.Xattr2, obj.getDisplay2().toString());
        if (obj.getVocabulary()!=null)
          attribute(XMLTags.Xtable, obj.getVocabulary().getVisibleName());
        attribute(XMLTags.Xpopup, obj.getPopupName());
        attribute(XMLTags.Xcomment, obj.getComment());
        endTag(); //parameter type
      }
    }
    endTag(); //parameter types

    startTag(XMLTags.XCommonParameters);
    plist = session.getAllObjectsForTechnology(CommonParameter.class,
        techn);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        CommonParameter obj = (CommonParameter) piter.next();
        startTag(XMLTags.XParameter);
        attribute(XMLTags.Xname, obj.getLongName());
        attribute(XMLTags.Xupload, obj.getVisualUploadName());
        attribute(XMLTags.Xshortname, obj.getVisibleName());
        attribute(XMLTags.Xfilter, obj.isShowInFilter() ? "1" : "0");
        attribute(XMLTags.Xcomment, obj.getComment());
        String uvz = obj.getAssayColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag(XMLTags.XAssayConf);
          attribute(XMLTags.Xcolumn, uvz);
          attribute(XMLTags.Xcol_numb, obj.getAssayViewColumnNumber().toString());
          attribute(XMLTags.Xsortable, obj.isAssaySortable() ? "1" : "0");
          endTag();
        }
        uvz = obj.getStgrColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag(XMLTags.XSGConf);
          attribute(XMLTags.Xcolumn, uvz);
          attribute(XMLTags.Xcol_numb, obj.getStgrViewColumnNumber().toString());
          attribute(XMLTags.Xsortable, obj.isStgrSortable() ? "1" : "0");
          endTag();
        }
        uvz = obj.getViewColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag(XMLTags.XViewConf);
          attribute(XMLTags.Xcolumn, uvz);
          attribute(XMLTags.Xcol_numb, obj.getViewViewColumnNumber().toString());
          attribute(XMLTags.Xsortable, obj.isViewSortable() ? "1" : "0");
          endTag();
        }
        endTag(); //Parameter
      }
    }
    endTag(); //Common parameters
    startTag(XMLTags.XParameters);
    plist = session.getAllObjectsForTechnology(Parameter.class, techn);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        Parameter obj = (Parameter) piter.next();
        startTag(XMLTags.XParameter);
        attribute(XMLTags.Xname, obj.getLongName());
        attribute(XMLTags.Xupload, obj.getVisualUploadName());
        attribute(XMLTags.Xshortname, obj.getVisibleName());
        attribute(XMLTags.Xgroup, obj.getParameterGroup().getVisibleName());
        attribute(XMLTags.Xtype, obj.getParameterType().getVisibleName());
        attribute(XMLTags.Xfilter, obj.isShowInFilter() ? "1" : "0");
        attribute(XMLTags.Xrequired, obj.isRequired() ? "1" : "0");
        attribute(XMLTags.Xnote, obj.getNoteToDisplay());
        attribute(XMLTags.Xcomment, obj.getComment());
        String uvz = obj.getAssayColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag(XMLTags.XAssayConf);
          attribute(XMLTags.Xcolumn, uvz);
          attribute(XMLTags.Xcol_numb, obj.getAssayViewColumnNumber().toString());
          attribute(XMLTags.Xsortable, obj.isAssaySortable() ? "1" : "0");
          attribute(XMLTags.Xrow_numb, obj.getAssayRowNumber().toString());
          endTag();
        }
        uvz = obj.getStgrColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag(XMLTags.XSGConf);
          attribute(XMLTags.Xcolumn, uvz);
          attribute(XMLTags.Xcol_numb, obj.getStgrViewColumnNumber().toString());
          attribute(XMLTags.Xsortable, obj.isStgrSortable() ? "1" : "0");
          attribute(XMLTags.Xrow_numb, obj.getStgrRowNumber().toString());
          endTag();
        }
        uvz = obj.getViewColumnName();
        if (uvz != null && !uvz.equals("")) {
          startTag(XMLTags.XViewConf);
          attribute(XMLTags.Xcolumn, uvz);
          attribute(XMLTags.Xcol_numb, obj.getViewViewColumnNumber().toString());
          attribute(XMLTags.Xsortable, obj.isViewSortable() ? "1" : "0");
          attribute(XMLTags.Xrow_numb, obj.getViewRowNumber().toString());
          endTag();
        }
        endTag(); //Parameter
      }
    }
    endTag(); //Parameters
    startTag(XMLTags.XDataFileTypes);
    plist = session.getAllObjectsForTechnology(DataType.class, techn);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        DataType obj = (DataType) piter.next();
        startTag(XMLTags.XFileType);
        attribute(XMLTags.Xtype, obj.getVisibleName());
        attribute(XMLTags.Xfile_ext, obj.getFileExtension());
        attribute(XMLTags.Xcomment, obj.getComment());
        endTag();
      }
    }
    endTag(); //data file types
    startTag(XMLTags.XProtocolFileTypes);
    plist = session.getAllObjectsForTechnology(ProtocolType.class, techn);
    if (plist != null) {
      Iterator piter = plist.iterator();
      while (piter.hasNext()) {
        ProtocolType obj = (ProtocolType) piter.next();
        startTag(XMLTags.XFileType);
        attribute(XMLTags.Xtype, obj.getVisibleName());
        attribute(XMLTags.Xfile_ext, obj.getFileExtension());
        attribute(XMLTags.Xcomment, obj.getComment());
        endTag();
      }
    }
    endTag(); //protocol file types
/*
    StatelessSession mySession = session.openStatelessSession();
    long technid = techn.getId().longValue();
    Query studyQuery = mySession.createQuery("select myGroup from StudyGroup as myGroup"
      + " where myGroup.technology = :technid");
    studyQuery.setLong( "technid", technid);
    startTag(XMLTags.XStudyGroups);
    List sgpList = session.getAllParametersSorted(technid, 1);
    List aspList = session.getAllParametersSorted(technid, 2);
    ScrollableResults studys = studyQuery.scroll(ScrollMode.FORWARD_ONLY);
    while (studys.next()) {
      startTag(XMLTags.Xstudy);
      StudyGroup myGroup = (StudyGroup) studys.get(0);
      attribute(XMLTags.Xname, myGroup.getVisibleName());

      Iterator sgpIterator = sgpList.iterator();
      while (sgpIterator.hasNext()) {
        Parameter param = (Parameter) sgpIterator.next();
        String fieldName = param.getStgrColumnNameF();
        if (fieldName != null && !fieldName.equals("")) {
          Method xyz = StudyGroup.class.getMethod("get" + fieldName, (Class[])null);
          String value = (String) xyz.invoke(myGroup, (Object[])null);
          if (value != null && !value.equals("")){
            ParameterType paramType = param.getParameterType();
            int baseType = paramType.getBaseType().intValue();
            if (baseType == AssayTypeName.TABLE) {
              Vocabulary vocab = paramType.getVocabulary();
              if (vocab != null) value = vocab.getValueById(value);
            }
//            String myValue = Util.getHTMLString(value);
            attribute(fieldName, value);
          }
        }
      }
      long groupid = myGroup.getId().longValue();
      Query assayQuery = mySession.createQuery("select myAssay from Assay as myAssay"
        + " where myAssay.technology = :technid and myAssay.studyGroup = :groupid ");
      assayQuery.setLong( "technid", technid);
      assayQuery.setLong("groupid", groupid);
      startTag(XMLTags.XAssays);
      ScrollableResults assays = assayQuery.scroll(ScrollMode.FORWARD_ONLY);
      while (assays.next()) {
        startTag(XMLTags.Xassay);
        Assay myAssay = (Assay) assays.get(0);
        attribute(XMLTags.Xname, myAssay.getVisibleName());
        Iterator aspIterator = aspList.iterator();
        while (aspIterator.hasNext()) {
          Parameter param = (Parameter) aspIterator.next();
          String fieldName = param.getStgrColumnNameF();
          if (fieldName != null && !fieldName.equals("")) {
            Method xyz = Assay.class.getMethod("get" + fieldName, (Class[])null);
            String value = (String) xyz.invoke(myAssay, (Object[])null);
            if (value != null && !value.equals("")){
              ParameterType paramType = param.getParameterType();
              int baseType = paramType.getBaseType().intValue();
              if (baseType == AssayTypeName.TABLE) {
                Vocabulary vocab = paramType.getVocabulary();
                if (vocab != null) value = vocab.getValueById(value);
              }
//              String myValue = Util.getHTMLString(value);
              attribute(fieldName, value);
            }
          }
        }
        endTag();
      }
      assays.close();
      endTag();//assays
      endTag();
    }
    studys.close();
    endTag(); //studyGroups
    mySession.close();
*/    
    endTag(); //technology
  }
  endTag();//AIMS_Export
}

}
