package assays.export;
/*
 Main class for xml/txt export utility
arguments: 
	1) UserHttpSession
	2) path to the tomcat SIMS application
	3) path to the config.xml file 
	4) technology id
  	5) list of study group ids (id1,id2,id3...)
  			or 
  	   studyView id
  	6) "list" or "view" - define what is in the fifth argument 
     
Output: getOutputStream(true) - result is in TSV format, getOutputStream(false) - result is in XML format 

 */
import org.w3c.dom.*;
import org.apache.log4j.Logger;

import org.hibernate.*;

import javax.xml.parsers.*; 

import java.util.*;
import java.io.*;
import java.nio.channels.*;

import assays.hibernate.*;
import assays.com.*;


public class AssayExport {
	private static UserHttpSession session;
	public static Logger log=org.apache.log4j.Logger.getLogger("simple");
	private static Session session1;
	private static Session session2;
	public static Technology tech;
//	public static StudyView view = null;
	private static Map<String, Integer> map;    // hash table
	private static int npk=0;
	private static String configFile;
	private static String techID = null;
        private static String assList = null;
	private static String sgList = null;
        private static String viewList = null;
	private static String viewID = null;
	
	//Java reflections
	private static ReflectionMethod rfTechnology = new ReflectionMethod("assays.hibernate.Technology");
	private static ReflectionMethod rfAssay      = new ReflectionMethod("assays.hibernate.Assay");
	private static ReflectionMethod rfStudyGroup = new ReflectionMethod("assays.hibernate.StudyGroup");
	private static ReflectionMethod rfStudyView  = new ReflectionMethod("assays.hibernate.StudyView");
  //      private static ReflectionMethod rfViewStudyMembership = new ReflectionMethod("assays.hibernate.ViewStudyMembership");

	private static ReflectionMethod rfBottom = new ReflectionMethod("sims.hibernate.Bottom");
	private static ReflectionMethod rfMiddle = new ReflectionMethod("sims.hibernate.Middle");
	private static ReflectionMethod rfTop    = new ReflectionMethod("sims.hibernate.Top");


//==========================================================================================================================
//                          New version for all levels ( 0 - views, 1 - study groups, 2 - assays
//==========================================================================================================================

	public AssayExport(UserHttpSession userSess, String confFile, String techVal, String list, int level){

        try{
            //   log.setLevel(org.apache.log4j.Level.ALL);

               log.setLevel(org.apache.log4j.Level.WARN);

               log.warn("AssayExport " + techVal + " " + level + " " + list);

               switch (level)
               {
                   case 0: //view
                           session = userSess;
		           configFile = confFile;
		           
   		           map = new HashMap<String, Integer>();
   		           session1 = userSess.getAssaySess();
   		           session2 = userSess.getSampleSess();
		           tech = session.getCurrentTechnology();
                           techID = tech.getId().toString(); 
   		           viewList=list;
                           break;

                   case 1: //study group
                           session = userSess;
		           configFile = confFile;
		     
   		           map = new HashMap<String, Integer>();
   		           session1 = userSess.getAssaySess();
   		           session2 = userSess.getSampleSess();
		           tech = session.getCurrentTechnology();
                           techID = tech.getId().toString();    
   		           sgList=list;
                           break;

                   case 2: //assay
                           session = userSess;
		           configFile = confFile;

   		           map = new HashMap<String, Integer>();
   		           session1 = userSess.getAssaySess();
   		           session2 = userSess.getSampleSess();
		           tech = session.getCurrentTechnology();
                           techID = tech.getId().toString();
   		           assList=list;
                           break;
               }

   		}
   		catch (Exception e) {
			e.printStackTrace();
			log.error("AssayExport02 "+e.getMessage());
		}
	}
        
//==========================================================================================================================

	private static List getList(String selectStatement){
		List tasks = new Vector(); 
		try {
			tasks = session1.createQuery(selectStatement)
			.list();				
		} catch (HibernateException e) {
			e.printStackTrace();
			log.error("AssayExport04 "+e.getMessage());
		}
		return tasks;

}

//==========================================================================================================================

private static Object getValue(long simsRefID, int level){
	Object al = new Object();
	try {
		if (level == 1){
			al = (Object) session2.createQuery("select a from Top as a where a.id="+simsRefID).uniqueResult();
		}
		else 
			if (level == 2){
				al = (Object) session2.createQuery("select a from Middle as a where a.id="+simsRefID).uniqueResult();
			}
			else if (level == 3){
					al = (Object) session2.createQuery("select a from Bottom as a where a.id="+simsRefID).uniqueResult();
				}		
				
	} catch (HibernateException e) {
		e.printStackTrace();
		log.error("AssayExport05 "+e.getMessage()+" select a from Bottom/Middle/Top as a where a.id="+simsRefID);
	}
	return al;

}

//==========================================================================================================================

public static OutputStream getOutputStream(boolean txt){

  //  System.out.println(" <<<<<<<<<<< GOS >>>>>>>>>>>>>>>>>>> " + String.valueOf(txt) + " " + configFile);
  //  Util.writeLog("getOutputStream", String.valueOf(txt), configFile);
    log.info("getOutputStream "+String.valueOf(txt));

    ByteArrayOutputStream out = new ByteArrayOutputStream();
   	Element root_element;
	Unit[] elementList=new Unit[0];
	try{
        //Create instance of DocumentBuilderFactory
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        //Get the DocumentBuilder
        DocumentBuilder docBuilder = factory.newDocumentBuilder();
        //Create blank DOM Document
        Document doc = docBuilder.newDocument();
        Document docConfig = docBuilder.parse (new File(configFile));
        docConfig.getDocumentElement ().normalize ();
	    NodeList unitList = docConfig.getElementsByTagName("unit");
	    
	    elementList = new Unit[unitList.getLength()];
	    
	    //Start to process each particular UNIT tag (creation of attributes)
	    for(int s=0; s<unitList.getLength() ; s++){
            Node currentNode = unitList.item(s);
            	Element currentUnit = (Element)currentNode;
                Element parent = (Element)currentUnit.getParentNode();
                //Create a list with attributes of type "const"
                NodeList attrList = currentUnit.getElementsByTagName("const");
                //Create a list with attributes of type "db"
                NodeList dbList = currentUnit.getElementsByTagName("db");
                //Create a list with attributes of type "func"
                NodeList funcList = currentUnit.getElementsByTagName("fn");
                //Create a list with attributes of type "funcR"
                NodeList funcRList = currentUnit.getElementsByTagName("fnRecursive");
                
                //All possible attributes will be added into this Vector as objects
                Vector<Attribute> attrVal = new Vector<Attribute>();
                //Add "const" attributes  
                for(int a=0; a<attrList.getLength() ; a++){
                     	Element currentAttr = (Element)attrList.item(a);
                     	if (currentAttr.getParentNode()==currentNode){
                     		//Create an Attribute object for type "const"
                     		Attribute attrObject = new Attribute(currentAttr.getAttribute("name"),currentAttr.getAttribute("value"),session.getHttpSessId());//log
                     		attrVal.addElement(attrObject);
                     	}   
                }
                //Add "db" attributes 
                for(int b=0; b<dbList.getLength() ; b++){
                     	Element currentAttr = (Element)dbList.item(b);
                     	if (currentAttr.getParentNode()==currentNode){
                     		//Create an Attribute object for type "db"
                     		Attribute attrObject = new Attribute(currentAttr.getAttribute("name"),currentAttr.getAttribute("col"),currentAttr.getAttribute("format"),"db",session.getHttpSessId());//log
                     		attrVal.addElement(attrObject);
                     	}
               }
                //Add "func" attributes 
                for(int c=0; c<funcList.getLength() ; c++){
                 	Element currentAttr = (Element)funcList.item(c);
                 	if (currentAttr.getParentNode()==currentNode){
                 		//Create an Attribute object for type "func"
                 		Attribute attrObject = new Attribute(currentAttr.getAttribute("name"),currentAttr.getAttribute("conv"),currentAttr.getAttribute("param"),"func",session.getHttpSessId());//log
                 		attrVal.addElement(attrObject);                     		
                 	}
                }
                //Add "funcR" attributes 
                for(int d=0; d<funcRList.getLength() ; d++){
                 	Element currentAttr = (Element)funcRList.item(d);
                 	if (currentAttr.getParentNode()==currentNode){
                 		//Create an Attribute object for type "func"
                 		Attribute attrObject = new Attribute(currentAttr.getAttribute("name"),"*"+currentAttr.getAttribute("conv"),currentAttr.getAttribute("param"),"func",session.getHttpSessId());//log
                 		attrVal.addElement(attrObject);                     		
                 	}
                }
                //Create new element UNIT
                Unit strElement = new Unit(currentUnit.getAttribute("tag"),parent.getAttribute("tag"),currentUnit.getAttribute("filter"),attrVal,currentUnit.getAttribute("table"));
                //Add new element into a list of all elements
                elementList[s]=strElement;
                
                //Testam
             //   System.out.println ("Unit "+s+" "+elementList[s].name);
             //   strElement.printStructureUnit();

                }

            log.warn("Before root");

        //Create the root element
        root_element = doc.createElement(docConfig.getDocumentElement().getAttribute("tag"));

             log.warn("After root");
        
        //Create XML structure (start of recursive procedure)
        createXMLUnit(doc, elementList, root_element, new StudyView(), new StudyGroup(), new Assay(), new Object(), "", "");

           log.warn("After create XMLUnit");
       
 //        System.out.println(" <<<<<<<<<<< GOS-2 >>>>>>>>>>>>>>>>>>> " + String.valueOf(txt));
//    Util.writeLog("getOutputStream", String.valueOf(txt), configFile);
//    log.info("getOutputStream2 "+String.valueOf(txt));
        
        // Format and Serialize the document 
        if (txt){
        	DataOutputStream dos = new DataOutputStream(out);
        	if (configFile.contains("idf")) {
        		TSV txtOS = new TSV(doc.getDocumentElement(),"planar",dos);
        	}
        	else {
        		TSV txtOS = new TSV(doc.getDocumentElement(),dos);
        	}
        }
        else {
        	org.apache.xml.serialize.OutputFormat format = new org.apache.xml.serialize.OutputFormat(doc);

                log.warn("After format");


        	format.setLineWidth(100);
        	format.setIndenting(true);
        	format.setIndent(2);
        	format.setOmitComments(true);
        //	System.out.println(configFile);
        	if (configFile.contains("DW_")) {
        		format.setDoctype(null, "AEDW.dtd");
        	}

        	//Result xml file creation from DOM document
        	org.apache.xml.serialize.XMLSerializer ser= new org.apache.xml.serialize.XMLSerializer(out, format);

                log.warn("After ser");

        	ser.serialize(doc);

                log.warn("After serialize");
      // Testam
      //          System.out.println("ser rez="+ser.toString());
      //          Util.writeLog("ser rez=",ser.toString(),"");
        }
        
        }catch(Exception e){
        	e.printStackTrace();
        	log.error("AssayExport06 "+e.getMessage());
        }
        
        return out;  
}

//==========================================================================================================================

//Method to create new DOM element
private static Element createDOMElement(Document doc,String name,Vector attr,String tableName, StudyView sv, StudyGroup sg, Assay a,Object alq, Element parent){

   //log.info("createDOMElement ");
   log.warn(" >>DOM \"" + name + "\" \"" + tableName +"\" f:" + Runtime.getRuntime().freeMemory());

	//Create new DOM element
	Element childElement = doc.createElement(name);
	String constVal="";
	String constName="";
    String valueVal="";
    Vector valueArray = new Vector();
    boolean xmlBlock = false;
    String methodName;
    boolean paramString=false;
    for(int i = 0; i < attr.size(); i++) {
    	valueVal="";
    	Attribute currAttr=(Attribute) attr.get(i);
    	//Find value for attribute with type "const" 
    	if (currAttr.type.equalsIgnoreCase("const")){
    		valueVal=currAttr.val;
			if (valueVal.contains("npkID")){
				npk=npk+1;
				valueVal=valueVal.replace("npkID", Integer.toString(npk));
			}
    		constVal=valueVal;
    		constName=currAttr.name;
    	}
    	else {
    		//Find value for attribute with type "func" 
    		if (currAttr.type.equalsIgnoreCase("func")){
    			valueVal="";
    			Vector<String> paramVal = new Vector<String>();
    			try{
    		        //System.out.println("Assay:"+a.getId());
    			//System.out.println("Size:"+currAttr.param.size());
    			//System.out.println("FN:"+currAttr.format);
    			for(int b=0; b<currAttr.param.size() ; b++){
    				 String p = (String)currAttr.param.get(b);        	    				 
    				 paramString=true;
    				 if (p.length()>3){
    					if (p.substring(0,3).equals("db_")){
    						 //DB field to find a value
    						 methodName=p.substring(3);
    						 //Find DB value for function arguments
    						 String res = getDBValue(tableName,methodName,sv,sg,a,alq);
    						 //DB value for function arguments is passed into Attribute object param[]
    						  paramVal.add(res);
   							 paramString=false;
    					 }   
    					 if (p.length()>5){ 
    						 if (p.substring(0,5).equals("conf_")){
    							 //config file attribute to find a value
    							 methodName=p.substring(5);
    							 //Find config attribute
    							 String res = HibernateSessionFactory.getProperty(methodName);
    							
    							 paramVal.add(res);
    							 paramString=false; 
    						 }
    					 }
    				 }
    				 if (paramString) {paramVal.add(p);}
    			 }
    			 
    			if (currAttr.format.substring(0,1).equals("*")){
    				
    				valueArray=currAttr.getFuncValArray(currAttr.format.substring(1),paramVal);
    				xmlBlock = true;
    			}
    			else
    				valueVal=currAttr.getFuncVal(paramVal); //Function is processed and result is stored as value of an attribute
    			}
    			 catch (HibernateException e) {
    				 e.printStackTrace();
     			 	log.error("AssayExport07 "+e.getMessage());
     			  }   
    		}
    		//Find value for attribute with type "db" 
    		else{
    			methodName=currAttr.col;        			
    			currAttr.setVal(getDBValue(tableName,methodName,sv,sg,a,alq));
    			valueVal=currAttr.getVal();
    		}
    	}
    	//Add attribute for a created DOM element
    	if (!xmlBlock)
    		childElement.setAttribute(currAttr.name,valueVal);
    	else {
     		childElement.setAttribute(currAttr.name,(String)valueArray.get(0));
    		for(int b=1; b<valueArray.size(); b++) {
    			  if (doc.hasChildNodes())
    		        	parent.appendChild(childElement);
    		        else {
    		        	doc.appendChild(childElement);
    		        }
    			childElement = doc.createElement(name);
    			childElement.setAttribute(constName,constVal);
    			childElement.setAttribute(currAttr.name,(String)valueArray.get(b));
    		}
    	}
    }  
    if (doc.hasChildNodes())
    	parent.appendChild(childElement);
    else {
    	doc.appendChild(childElement);
    }
    
    
    log.warn(" <<DOM f:" + Runtime.getRuntime().freeMemory());
    return childElement; //return created DOM element with attributes
}

//==========================================================================================================================

//Recursive method that goes through list of units and creates substructures and elements of DOM document 	
private static void createXMLUnit(Document doc, Unit[] strUnitVal, Element parent, StudyView sv, StudyGroup sg, Assay a, Object al, String tableNameVal, String listNameVal){
	Element ref_element = parent;
	Element new_element = parent;
	String current_name = parent.getNodeName();
  //      ViewStudyMembership currentViewStudyMembership = vsm;
        StudyView currentStudyView = sv;
	StudyGroup currentStudyGroup=sg;
	Assay currentAssay=a;
	Object currentAliquot=al;
	String tableName=tableNameVal;
	List currentList = new Vector();
	boolean isList=false;
	Unit[] strUnitNew = new Unit[0];
	String listName = listNameVal;
	int simsLevel = 0;
	int cnt;

  //      log.info(" <<<<<<<<<<< CreateXMLUnit >>>>>>>>>>>>>>>>>>> \"" + tableNameVal + "\" \"" + listNameVal +"\"");
        log.warn(" >>CreateXMLUnit \"" + tableNameVal + "\" \"" + listNameVal +"\" f:" + Runtime.getRuntime().freeMemory());
        
        strUnitVal[0].printStructureUnit();
	
	if (a!=null)
        {
            log.info(" <<<<<<<<<<< CreateXMLUnit >>>>>>> A >>>>>>>>>> ");
            simsLevel=a.getSimsReferenceType();
        }
   else if (sg!=null)
        {      
            log.info(" <<<<<<<<<<< CreateXMLUnit >>>>>>>> SG >>>>>>>>>");
            simsLevel=sg.getSimsReferenceType();
        }
  else if (sv!=null)
        {
            log.info(" <<<<<<<<<<< CreateXMLUnit >>>>>>>> SV >>>>>>>>>");
            simsLevel=sv.getSimsReferenceType();
        }

		//(*)
		//First unit in list is substructure (unit will be a parent for number of other elements obtained from select statement)
		if (strUnitVal[0].list){
			//Store the name of current substructure
			listName=strUnitVal[0].name;
			String selectString = strUnitVal[0].selectStatement;
			//Execute select statement
			if (strUnitVal[0].selectStatement.contains("aID")){
				selectString=selectString.replace("aID", currentAssay.getId().toString());
			}
			if (strUnitVal[0].selectStatement.contains("sgID")){
				selectString=selectString.replace("sgID", currentStudyGroup.getId().toString());
			}
			if (strUnitVal[0].selectStatement.contains("viewID")){
				selectString=selectString.replace("viewID", currentStudyView.getId().toString());
			}
                        if (strUnitVal[0].selectStatement.contains("assList")){
				selectString=selectString.replace("assList", assList);
			}
			if (strUnitVal[0].selectStatement.contains("sgList")){
				selectString=selectString.replace("sgList", sgList);
			}
                        if (strUnitVal[0].selectStatement.contains("viewList")){

                            log.info("viewList: "+viewList);

				selectString=selectString.replace("viewList", viewList);
			}
			
			currentList=getList(selectString);
			
			//Possible values: "StudyGroup", "Assay", "Top", "Middle", "Bottom" ...
			tableName=strUnitVal[0].table;

                        log.info("After replace: tableName/select "+tableName+" "+selectString);

			
			if (strUnitVal[0].selectStatement.contains("simsRefID")){
				try{
					String simsRefID = "";
					Object[] arguments = new Object[0];
					if (simsLevel==1){
						simsRefID = (String) rfTop.getMethod("getId").invoke(currentAliquot, arguments).toString();	
					}
					if (simsLevel==2){
						simsRefID = (String) rfMiddle.getMethod("getId").invoke(currentAliquot, arguments).toString();	
					}
					if (simsLevel==3){
						simsRefID = (String) rfBottom.getMethod("getId").invoke(currentAliquot, arguments).toString();	
					}
					selectString=selectString.replace("simsRefID", simsRefID);
			    } catch (Exception e) {
			        log.error("AssayExport08 "+e.getMessage());
			    } 
			}

                        log.info("After SIMSreplace: tableName/select "+tableName+" "+selectString);
			
			//DOM Element (first element in new substructure) is first one after root or is at the same level as its parent
			if (current_name.equals(strUnitVal[0].parent) || !doc.hasChildNodes()){
	        	new_element=createDOMElement(doc,strUnitVal[0].name,strUnitVal[0].attr,tableName, currentStudyView,currentStudyGroup,currentAssay,currentAliquot,ref_element);
		     	ref_element=new_element;
		     	current_name=new_element.getNodeName();
		    }
			//DOM element (first element in new substructure) is inside of other substructure 
			else {
				//Search for the right parent element (ref_element)
	        	while (!current_name.equals(strUnitVal[0].parent)){		        		
	        			current_name=new_element.getNodeName();
	        			ref_element=new_element;
	        			if (!doc.getDocumentElement().getNodeName().equals(current_name))
	        				new_element=(Element) new_element.getParentNode();
		        	}
		     	new_element=createDOMElement(doc,strUnitVal[0].name,strUnitVal[0].attr,tableName, currentStudyView,currentStudyGroup,currentAssay,currentAliquot,ref_element);
		     	ref_element=new_element;
			}
		    
			//List elements from select statement and create elements of substructure
			Iterator iter = currentList.iterator();				 
			for(int i=0; i<currentList.size();i++){

                                  if (tableName.equals("ViewStudyMembership")){
					 //Define current ViewStudyMembership
			//		 currentViewStudyMembership = (ViewStudyMembership) iter.next();
                                         currentStudyGroup = (StudyGroup) iter.next();

                                         log.info("UzstādaVSM currentStudyGroup="+currentStudyGroup.getId().toString());

				 }
                                 if (tableName.equals("StudyView")){
					 //Define current Study View
					 currentStudyView = (StudyView) iter.next();

                                         log.info("Uzstāda currentStudyView="+currentStudyView.getId().toString());

				 }
				 if (tableName.equals("StudyGroup")){
					 //Define current Study Group 
					 currentStudyGroup = (StudyGroup) iter.next();

                                         log.info("UzstādaSG currentStudyGroup="+currentStudyGroup.getId().toString());
				 }
				 if (tableName.equals("Assay")){
					 //Define current Assay
					 currentAssay = (Assay) iter.next();

                                         log.info("Uzstāda currentAssay="+currentAssay.getId().toString());
				 }
				 if (tableName.equals("Bottom")||tableName.equals("Middle")||tableName.equals("Top")){
					 //Define current Aliquot
					 currentAliquot = (Object) getValue(Long.parseLong(iter.next().toString()),simsLevel);
					 
				 }
				 //If substructure consists of another substructres then correct parent element is needed
				 //(**)
				 //Check is second element in new substructure has parent a first element
				 //or parent of just created new element is root
				 //in that case new element is created where parent is previously created first element of substructure
				 if (current_name.equals(strUnitVal[1].parent) || new_element.getParentNode()==doc.getDocumentElement()){
			        	new_element=createDOMElement(doc,strUnitVal[1].name,strUnitVal[1].attr,tableName, currentStudyView,currentStudyGroup,currentAssay,currentAliquot,ref_element);
				 }
			     else {			
			     //Otherwise (see **) search for the right parent element (ref_element)
			        	while (!current_name.equals(strUnitVal[1].parent)){
			        		current_name=new_element.getNodeName();
		        			ref_element=new_element;
		        			if (!doc.getDocumentElement().getNodeName().equals(current_name))
		        				new_element=(Element) new_element.getParentNode();
			        	}
				     	new_element=createDOMElement(doc,strUnitVal[1].name,strUnitVal[1].attr,tableName, currentStudyView,currentStudyGroup,currentAssay,currentAliquot,ref_element);
			     }
				 //Create new list of units - through out first two elements
				 strUnitNew = new Unit[strUnitVal.length-2];
				 for (int j=2; j<strUnitVal.length; j++){
					 strUnitNew[j-2]=strUnitVal[j];
					 
				 }
				 //Call for method with new arguments
				 createXMLUnit(doc, strUnitNew,new_element,currentStudyView,currentStudyGroup,currentAssay,currentAliquot,tableName,listName);
			}
		}
		//If first element in a list of units is not a new substructure - stay at the same level of DOM (see *)
		else {
			 cnt=0;
			 isList=false;
			 //Create new DOM elements until element which is list (new substructure) is found 
			 while ((!isList) && (cnt<strUnitVal.length)){
				 isList=strUnitVal[cnt].list;
				 if (!isList){
					 //DOM Element is first one after root or is at the same level as its parent	 
				 if (current_name.equals(strUnitVal[cnt].parent) || !doc.hasChildNodes()){
		        	new_element=createDOMElement(doc,strUnitVal[cnt].name,strUnitVal[cnt].attr,tableName,currentStudyView,currentStudyGroup,currentAssay,currentAliquot,ref_element);
			     	if (!doc.hasChildNodes()){
			     		ref_element=new_element;	
			     		current_name=new_element.getNodeName();
			     	}
				 }
				 else {
					 //	Search for the right parent element (ref_element) 
		        	while (!current_name.equals(strUnitVal[cnt].parent)){
		        		current_name=new_element.getNodeName();
	        			ref_element=new_element;
	        			if (!doc.getDocumentElement().getNodeName().equals(current_name))
	        				new_element=(Element) new_element.getParentNode();
		        	}
			     	new_element=createDOMElement(doc,strUnitVal[cnt].name,strUnitVal[cnt].attr,tableName,currentStudyView,currentStudyGroup,currentAssay,currentAliquot,ref_element);
				 }
				 cnt++;
				 }//end of if !isList
			 }//end of while
			 
			 //if current unit element is a new substructure - process it
			 if (cnt<strUnitVal.length && strUnitVal[cnt].list) {
				 	//Create new list of units - through out all processed elements
				 	strUnitNew = new Unit[strUnitVal.length-cnt];
				 	for (int j=cnt; j<strUnitVal.length; j++){
				 		strUnitNew[j-cnt]=strUnitVal[j];
				 	}
				 	//Check if a new list (substructure) is sublist of previous one list (substructure)
				 	String parent_name=strUnitNew[0].parent;					 	
				 	boolean subList=false;
				 	Element current_element = (Element) doc.getFirstChild();
				 	current_name = current_element.getNodeName();
				 	while (!subList && !current_name.equals(parent_name)){
					 	if (current_element!=null){
					 		current_name=current_element.getNodeName();
					 		ref_element=current_element;
					 		current_element=(Element) current_element.getFirstChild();
					 	}
					 	else subList=true;	
					 	if (current_name.equals(listName)) subList=true;						 	
		        	}
				 	
				 	
				 if (subList) createXMLUnit(doc, strUnitNew,new_element,currentStudyView,currentStudyGroup,currentAssay,currentAliquot,tableName,listName);
				 else 	
					 //if a new list is not a sublist check if the list (substructure) have been already created in previous recursion level 
					 if (!map.containsKey(strUnitNew[0].name)){
						 map.put(strUnitNew[0].name, new Integer(1)); //put such kind of list into a hash table 
						 //and create it in DOM document
						 createXMLUnit(doc, strUnitNew,new_element,currentStudyView,currentStudyGroup,currentAssay,currentAliquot,tableName,listName);
					 }					 
			 }
		}// end if first element in a list of units is not a new substructre


        strUnitNew = null;
        currentList = null;

        log.warn(" <<CreateXMLUnit \"" + tableNameVal + "\" \"" + listNameVal +"\" f:" + Runtime.getRuntime().freeMemory());


}

//==========================================================================================================================

//Work with DB - using Java reflection dynamically load methods from Hibernate classes 
//(Assay,StudyGroup,Bottom,Middle,Top,MiddleSubType) and return DB values	
private static String getDBValue(String tableName, String methodNameVal, StudyView sv, StudyGroup sg, Assay a, Object simsRefObject){
	Parameter paramType;
	int baseType;
	Object[] arguments = new Object[0];
	String result="";
	String methodName = methodNameVal;
	int simsLevel = 0;
	Object TopObject = null;
		Object MiddleObject = null;
		Object BottomObject = null;
		boolean isPersistent = false;
		String refTableName = tableName;

         //       log.info("<<<<<< getDBValue >>>>>>> " + refTableName + "/" + methodName + " result=" + result);
                log.warn(" >>getDBValue " + refTableName + "/" + methodName + " f:" + Runtime.getRuntime().freeMemory());

		
		if (methodName.contains(".")){
			refTableName=methodName.substring(0, methodName.indexOf("."));
			methodName=methodName.substring(methodName.indexOf(".")+1);
			methodName="get"+methodName.substring(0, 1).toUpperCase()+methodName.substring(1);
		}
		else {
			methodName="get"+methodName.substring(0, 1).toUpperCase()+methodName.substring(1);
		}

                log.info("Pirms try: refT="+refTableName+" T="+tableName + " result=" + result);

   	try{
 	 //References to SIMS if main table is StudyView
 		if ((tableName.equals("StudyView")) && (sv != null)){
 			simsLevel = sv.getSimsReferenceType();
                        
                        log.info("getDBValue sv "+String.valueOf(simsLevel) + " result=" + result);
                        
 			if (simsLevel==1){
					TopObject = (Object) session2.createQuery("select a from Top as a where a.id='"+ sv.getSimsTopRef() + "'").uniqueResult();//(Object) view.getSimsTopRef();
				}
				if (simsLevel==2){
					MiddleObject = (Object) session2.createQuery("select a from Middle as a where a.id='"+ sv.getSimsMiddleRef() + "'").uniqueResult();//(Object) view.getSimsMiddleRef();
					TopObject = rfMiddle.getMethod("getParent").invoke(MiddleObject, arguments);
				}
				if (simsLevel==3){
					BottomObject = (Object) session2.createQuery("select a from Bottom as a where a.id='"+ sv.getSimsBottomRef() + "'").uniqueResult();//(Object) view.getSimsBottomRef();
					MiddleObject = rfBottom.getMethod("getParent").invoke(BottomObject, arguments);
					TopObject = rfMiddle.getMethod("getParent").invoke(MiddleObject, arguments);
				}
							
			}



              log.info("Pass A  refT="+refTableName+" T="+tableName + " result=" + result);

 		//References to SIMS if main table is StudyGroup
 		if (tableName.equals("StudyGroup")){

                   

 			simsLevel = sg.getSimsReferenceType();
                        
                     log.info("getDBValue StudyGroup "+String.valueOf(simsLevel) + " result=" + result);
                        
 			if (simsLevel==1){
					TopObject = (Object) session2.createQuery("select a from Top as a where a.id='"+ sg.getSimsTopRef() + "'").uniqueResult();//(Object) sg.getSimsTopRef();
				}
				if (simsLevel==2){
					MiddleObject =  (Object) session2.createQuery("select a from Middle as a where a.id='"+ sg.getSimsMiddleRef() + "'").uniqueResult();//(Object) sg.getSimsMiddleRef();
					TopObject = rfMiddle.getMethod("getParent").invoke(MiddleObject, arguments);
				}
				if (simsLevel==3){
					BottomObject =  (Object) session2.createQuery("select a from Bottom as a where a.id='"+ sg.getSimsBottomRef() + "'").uniqueResult();//(Object) sg.getSimsBottomRef();
					MiddleObject = rfBottom.getMethod("getParent").invoke(BottomObject, arguments);
					TopObject = rfMiddle.getMethod("getParent").invoke(MiddleObject, arguments);
				}
							
			}


                log.info("Pass B  refT="+refTableName+" T="+tableName + " result=" + result);

 		//References to SIMS if main table is Assay
 		if (tableName.equals("Assay")){
 			simsLevel = a.getSimsReferenceType();
                        
                        log.info("getDBValue Assay "+String.valueOf(simsLevel));  
                        
 			if (simsLevel==1){
					TopObject =  (Object) session2.createQuery("select a from Top as a where a.id='"+ a.getSimsTopRef() + "'").uniqueResult();//(Object) a.getSimsTopRef();
				}
				if (simsLevel==2){
					MiddleObject =  (Object) session2.createQuery("select a from Middle as a where a.id='"+ a.getSimsMiddleRef() + "'").uniqueResult();//(Object) a.getSimsMiddleRef();
					TopObject = rfMiddle.getMethod("getParent").invoke(MiddleObject, arguments);
				}
				if (simsLevel==3){
					BottomObject =  (Object) session2.createQuery("select a from Bottom as a where a.id='"+ a.getSimsBottomRef() + "'").uniqueResult();//(Object) a.getSimsBottomRef();
					MiddleObject = rfBottom.getMethod("getParent").invoke(BottomObject, arguments);
					TopObject = rfMiddle.getMethod("getParent").invoke(MiddleObject, arguments);
				}
						
			}


                log.info("Pass C0  refT="+refTableName+" T="+tableName + " result=" + result);

 		//References to SIMS if main table is Bottom, Middle or Top


                
 		if (tableName.equals("Bottom")||tableName.equals("Middle")||tableName.equals("Top")){

                    log.info("Esam B/M/T daļā "+tableName);

                    //al=simsRefObject;
 			simsLevel = a.getSimsReferenceType();
                        
                        log.info("getDBValue "+tableName+"/"+String.valueOf(simsLevel) + " result=" + result);
                        
				if (simsLevel==1){
					TopObject = simsRefObject;
				}
				if (simsLevel==2){
					MiddleObject = simsRefObject;
					TopObject = rfMiddle.getMethod("getParent").invoke(MiddleObject, arguments);
				}
				if (simsLevel==3){
					BottomObject = simsRefObject;
					MiddleObject = rfBottom.getMethod("getParent").invoke(BottomObject, arguments);
					TopObject = rfMiddle.getMethod("getParent").invoke(MiddleObject, arguments);
				}
			}

                 
                log.info("Pass C  refT="+refTableName+" T="+tableName + " result=" + result);
                
 		//Methods for referenced tables
 		baseType=0;
 		//StudyView methods
 		if (refTableName.equals("StudyView") && (sv != null)){
                    
                      log.info("getDBValue refTableName="+refTableName+" methodName="+methodName + " result=" + result);



                      log.info("getDBValue Query="+"select myParam from Parameter as myParam where myParam.technology = '"
			          	+ techID + "' and "
			          	+ "myParam.viewColumnName = '"
			          	+ methodName.substring(3)+ "'");
                    
  			if (methodName.substring(3,4).toUpperCase().equals("F")){
				paramType=(Parameter) session1.createQuery("select myParam from Parameter as myParam where myParam.technology = '"
			          	+ techID + "' and "
			          	+ "myParam.viewColumnName = '"
			          	+ methodName.substring(3)+ "'").uniqueResult();
				baseType = paramType.getParameterType().getBaseType().intValue();
				//Vocabulary tables
				if (baseType==assays.hibernate.AssayTypeName.TABLE){
    				Vocabulary vocab = paramType.getParameterType().getVocabulary();
    				if (vocab!=null){
    					String myValue=(String) rfStudyView.getMethod(methodName).invoke(sv, arguments);
    					myValue = Util.getHTMLString(myValue);         		    				
    					result = vocab.getValueById(myValue);
    		      		}
    				else {
    					result = "";
    				}


                          log.info("getDBValue baseType="+baseType+"?="+assays.hibernate.AssayTypeName.TABLE+"  result="+result);

    			}
			}
  			if (baseType!=assays.hibernate.AssayTypeName.TABLE){ 
				isPersistent = false;
				if (methodName.equals("getId")){
					result = sv.getId().toString();
					isPersistent = true;
				}
				if (methodName.equals("getComment")){
					result = sv.getComment();
					isPersistent = true;
				}
				if (methodName.equals("getCreateDate")){
					result = sv.getCreateDate().toString();
					isPersistent = true;
				}
				if (methodName.equals("getModifDate")){
					result = sv.getModifDate().toString();
					isPersistent = true;
				}
				if (methodName.equals("getCreator")){
					result = sv.getCreator().getLoginName();
					isPersistent = true;
				}
				if (methodName.equals("getModifier")){
					result = sv.getModifier().getLoginName();
					isPersistent = true;
				}
				if (!isPersistent) {

                                        log.info("getDBValue pirms "+methodName);

					result =(String) rfStudyView.getMethod(methodName).invoke(sv, arguments);

                                        log.info("getDBValue pēc "+result);
					//System.out.println(methodName+result);
				}
  			} 
 		}


                log.info("Pass D  refT="+refTableName+" T="+tableName + " result=" + result);

 		//StudyGroup methods
 		if (refTableName.equals("StudyGroup") && (sg!=null)){

                     log.info("getDBValue StudyGroup "+methodName+" "+sg.getTechnology().getId());

 			if (methodName.substring(3,4).toUpperCase().equals("F")){
 				paramType=(Parameter) session1.createQuery("select myParam from Parameter as myParam where myParam.technology = '"
		          	+ sg.getTechnology().getId() + "' and "
		          	+ "myParam.stgrColumnName = '"
		          	+ methodName.substring(3)+ "'").uniqueResult();
 				baseType = paramType.getParameterType().getBaseType().intValue();
 				//Vocabulary tables
				if (baseType==assays.hibernate.AssayTypeName.TABLE){
    				Vocabulary vocab = paramType.getParameterType().getVocabulary();
    				if (vocab!=null){
    					String myValue=(String) rfStudyGroup.getMethod(methodName).invoke(sg, arguments);
    					myValue = Util.getHTMLString(myValue);         		    				
    					result = vocab.getValueById(myValue);
    		      		}
    				else {
    					result = "";
    				}
    			}
			}
  			if (baseType!=assays.hibernate.AssayTypeName.TABLE){ 
				isPersistent = false;
				if (methodName.equals("getId")){
					result = sg.getId().toString();
					isPersistent = true;
				}
				if (methodName.equals("getComment")){
					result = sg.getComment();
					isPersistent = true;
				}
				if (methodName.equals("getCreateDate")){
					result = sg.getCreateDate().toString();
					isPersistent = true;
				}
				if (methodName.equals("getModifDate")){
					result = sg.getModifDate().toString();
					isPersistent = true;
				}
				if (methodName.equals("getCreator")){
					result = sg.getCreator().getLoginName();
					isPersistent = true;
				}
				if (methodName.equals("getModifier")){
					result = sg.getModifier().getLoginName();
					isPersistent = true;
				}
				if (!isPersistent) {
					result =(String) rfStudyGroup.getMethod(methodName).invoke(sg, arguments);
				}
  			}     			
 		}


                log.info("Pass E  refT="+refTableName+" T="+tableName + " result=" + result);

 		//Assay methods
 		if (refTableName.equals("Assay") && (a!=null)){ 
                    
                    log.info("getDBValue Assay "+methodName+" "+a.getTechnology().getId());

                    log.info("getDBValue Query="+"select myParam from Parameter as myParam where myParam.technology = '"
	          	+ a.getTechnology().getId() + "' and "
	          	+ "myParam.assayColumnName = '"
	          	+ methodName.substring(3)+ "'");
                    
 			//F... fields types
 			if (methodName.substring(3,4).toUpperCase().equals("F")){
 				paramType=(Parameter) session1.createQuery("select myParam from Parameter as myParam where myParam.technology = '"
	          	+ a.getTechnology().getId() + "' and "
	          	+ "myParam.assayColumnName = '"
	          	+ methodName.substring(3)+ "'").uniqueResult();
 				baseType = paramType.getParameterType().getBaseType().intValue();
 				//Vocabulary tables
 				if (baseType==assays.hibernate.AssayTypeName.TABLE){
 					Vocabulary vocab = paramType.getParameterType().getVocabulary();
 					if (vocab!=null){
 						String myValue=(String) rfAssay.getMethod(methodName).invoke(a, arguments);
 						myValue = Util.getHTMLString(myValue);         		    				
 						result = vocab.getValueById(myValue);
 					}
 					else {
 						result = "";
 					}
 				}
 			}
 			if (baseType!=assays.hibernate.AssayTypeName.TABLE){ 
 				isPersistent = false;
 				if (methodName.equals("getId")){
 					result = a.getId().toString();
 					isPersistent = true;
 				}
 				if (methodName.equals("getComment")){
 					result = a.getComment();
 					isPersistent = true;
 				}
 				if (methodName.equals("getCreateDate")){
 					result = a.getCreateDate().toString();
 					isPersistent = true;
 				}
 				if (methodName.equals("getModifDate")){
 					result = a.getModifDate().toString();
 					isPersistent = true;
 				}
 				if (methodName.equals("getCreator")){
 					result = a.getCreator().getLoginName();
 					isPersistent = true;
 				}
 				if (methodName.equals("getModifier")){
 					result = a.getModifier().getLoginName();
 					isPersistent = true;
 				}
 				if (!isPersistent) {

                                         log.info("getDBValue pirms "+methodName);

 					result = (String) rfAssay.getMethod(methodName).invoke(a, arguments);

                                         log.info("getDBValue pēc "+result);
 				}
 			}
 		}

 	
                log.info("Pass F  refT="+refTableName+" T="+tableName + " result=" + result);

                //Technology methods
 		if (refTableName.equals("Technology")){   
				result = (String) rfTechnology.getMethod(methodName).invoke(sg.getTechnology(),arguments).toString();
			}
		
 		//Referenced table Bottom
			if (refTableName.equals("Bottom") && (BottomObject!=null)){
                          
				//System.out.println(methodName);
				result = (String) rfBottom.getMethod(methodName).invoke(BottomObject, arguments).toString();
			}
		//Referenced table Middle
		if (refTableName.equals("Middle") && (MiddleObject!=null)){
				result = (String) rfMiddle.getMethod(methodName).invoke(MiddleObject, arguments).toString();
		}
		//Referenced table Top
		if (refTableName.equals("Top") && (TopObject!=null)){
			result = (String) rfTop.getMethod(methodName).invoke(TopObject, arguments).toString();
		}


                log.info("Pass G result=" + result);

} catch (Exception e) {
    log.error("AssayExport09 "+e.getMessage()+" "+tableName+" "+refTableName+" "+methodName);
}

arguments = null;
paramType = null;
TopObject = null;
MiddleObject = null;
BottomObject = null;

log.warn(" <<getDBValue " + refTableName + "/" + methodName +" f:" + Runtime.getRuntime().freeMemory());

return result;
}

//==========================================================================================================================

private static File XMLFormat(File xmlFile){
	  try {
		  /*  Sets up a file reader to read the file passed on the command
         	line one character at a time */
		  File xmlFormatted = new File ("result.xml"); 		  
		  FileOutputStream fos = new FileOutputStream(xmlFormatted);
		  OutputStreamWriter osw = new OutputStreamWriter(fos, "UTF-8");
		  
		  /* Filter FileReader through a Buffered read to read a line at a time */
		  BufferedReader bufRead = new BufferedReader(new InputStreamReader(new FileInputStream(xmlFile), "UTF-8"));
		  String line;    // String that holds current file line
		  int tabs = 0;
		  // Read first two lines
		  line =  bufRead.readLine();
		  osw.write(line+"\n");
		  line =  bufRead.readLine();
		  osw.write(line+"\n");
		  line =  bufRead.readLine();
		  String tab_string="";
		  String line2="";
		  boolean firstTime=true;
		  // Read through file one line at time. Print line # and line
		  while (line != null){    			  	
			   	if (line.contains("/>")) {if (!firstTime)tabs--; else {line2 = bufRead.readLine(); if (line2.contains("</")) tabs--;} firstTime=false;}
			   	else firstTime=true;
			  	if (line.contains("</")) {tabs--;tabs--;}
			  	tab_string="";
				if (tabs>0) {
			  		int i=0;
			  		while (i<tabs){tab_string="\t"+tab_string;i++;}
			  		//System.out.println(tabs);
			  	}
			  	line=tab_string+line;
			  	osw.write(line+"\n");
			  	//System.out.println(line); 
			  	if (line2.length()>0) {line=line2; line2="";}
			  	else line = bufRead.readLine();
				tabs++;
		  }
		  
		  bufRead.close();
		  osw.close();
  	      return xmlFormatted;
	
	  }catch (IOException e){		  
		  e.printStackTrace();
		  	return xmlFile;
	  }

	}

//==========================================================================================================================

public static void copyFile(File in, File out) 
throws IOException 
{
FileChannel inChannel = new
    FileInputStream(in).getChannel();
FileChannel outChannel = new
    FileOutputStream(out).getChannel();
try {
    inChannel.transferTo(0, inChannel.size(),
            outChannel);    
} 
catch (IOException e) {
    throw e;
}
finally {
    if (inChannel != null) inChannel.close();
    if (outChannel != null) outChannel.close();
}
}

//==========================================================================================================================

}



