package assays.hibernate;

import java.util.*;
import assays.com.Util;

public abstract class AssayPersistent {
  private Long id;
  private String visibleName;
  private AssayUser creator;
  private AssayUser modifier;
  private Date createDate;
  private Date modifDate;
  private String comment;

  public Long getId() {
    return id;
  }

  public String getIdAsTitle(){
    return "title=\"Id="+this.getId().toString()+"\"";
  }
  public String getVisibleName() {
    return visibleName;
  }

  public AssayUser getCreator() {
    return creator;
  }

  public AssayUser getModifier() {
    return modifier;
  }

  public Date getCreateDate() {
    return createDate;
  }

  public Date getModifDate() {
    return modifDate;
  }

  public String getComment() {
    return comment;
  }

  public String getVisualComment() {
    return Util.getHTMLString(comment);
  }

  public String[] getValueArray(byte[] indexArray, int viewSize) {
    String[] valueArray = new String[viewSize];
    if (indexArray[AssayFieldname.VISIBLENAMEnum] > 0) {
      valueArray[indexArray[AssayFieldname.VISIBLENAMEnum] - 1] = visibleName;
    }
    if (indexArray[AssayFieldname.COMMENTnum] > 0) {
      valueArray[indexArray[AssayFieldname.COMMENTnum] - 1] = comment;
    }
    if (indexArray[AssayFieldname.CREATORnum] > 0) {
      valueArray[indexArray[AssayFieldname.CREATORnum] - 1] = creator.getLoginName();
    }
    if (indexArray[AssayFieldname.CREATEDATEnum] > 0) {
      valueArray[indexArray[AssayFieldname.CREATEDATEnum] - 1] =
          Util.truncateDate(createDate);
    }
    if (indexArray[AssayFieldname.MODIFIERnum] > 0) {
      valueArray[indexArray[AssayFieldname.MODIFIERnum] - 1] =
          (modifier != null)?modifier.getLoginName():"";
    }
    if (indexArray[AssayFieldname.MODIFDATEnum] > 0) {
      valueArray[indexArray[AssayFieldname.MODIFDATEnum] - 1] =
          Util.truncateDate(modifDate);
    }
    return valueArray;
  }

  public String[] getValueArrayForExport(byte[] indexArray, int viewSize) {
    return getValueArray( indexArray, viewSize);
  }

  public String getPropertyValue(String propertyName) {
    return
        (propertyName.equals(AssayFieldname.VISIBLENAME)) ? visibleName :
        (propertyName.equals(AssayFieldname.CREATOR)) ?
        (creator != null) ? creator.getLoginName() : "" :
        (propertyName.equals(AssayFieldname.CREATEDATE)) ? Util.truncateDate(createDate) :
        (propertyName.equals(AssayFieldname.MODIFIER)) ?
        (modifier != null) ? modifier.getLoginName() : "" :
        (propertyName.equals(AssayFieldname.MODIFDATE)) ? Util.truncateDate(modifDate) :
        (propertyName.equals(AssayFieldname.COMMENT)) ? comment :
        "";
  }

  public Object getPropertyObject(String propertyName) {
    return
        (propertyName.equals(AssayFieldname.VISIBLENAME)) ? (Object) visibleName :
        (propertyName.equals(AssayFieldname.CREATOR)) ? (Object) creator :
        (propertyName.equals(AssayFieldname.CREATEDATE)) ? (Object) createDate :
        (propertyName.equals(AssayFieldname.MODIFIER)) ? (Object) modifier :
        (propertyName.equals(AssayFieldname.MODIFDATE)) ? (Object) modifDate :
        (propertyName.equals(AssayFieldname.COMMENT)) ? (Object) comment :
        null;
  }

// -------------------------------------------
// -------------------------------------------
// -------------------------------------------
//method used by adding PendingExport's 
  public void setId(Long long1) {
    id = long1;
  }

  public void setVisibleName(String string) {
    visibleName = string;
  }

  public void setCreator(AssayUser user) {
    creator = user;
  }

  public void setModifier(AssayUser user) {
    modifier = user;
  }

  public void setCreateDate(Date date) {
    createDate = date;
  }

  public void setModifDate(Date date) {
    modifDate = date;
  }

  public void setComment(String string) {
    comment = string;
  }

  public void setCreateInfo(AssayUser user) {
    createDate = new Date();
    modifDate = new Date();
    creator = user;
    modifier = user;
  }

  public void setModifInfo(AssayUser user) {
    modifDate = new Date();
    modifier = user;
  }

  public String getCreateAndModifyString() {
    StringBuffer strBuf = new StringBuffer
        ("<tr><td>&nbsp;</td><td>Created by &nbsp;");
    strBuf.append(creator.getLoginName())
        .append("&nbsp; Create data &nbsp;")
        .append(Util.truncateTime(createDate))
        .append("</td></tr>");
    if (modifier != null) {
      strBuf.append("<tr><td>&nbsp;</td><td>Modified by &nbsp;")
          .append(modifier.getLoginName())
          .append("&nbsp; Modify data &nbsp;")
          .append(Util.truncateTime(modifDate))
          .append("</td></tr>");
    }
    return strBuf.toString();
  }

  public String toString() {
    return /*this.getId() + "  " + */this.getVisibleName();
  }

  public void CopyTo( AssayPersistent target){
    target.setComment(comment);
    target.setVisibleName(visibleName);
  }
}
