package assays.hibernate;

import java.io.StringReader;
import org.xml.sax.InputSource;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import java.util.Stack;
import org.w3c.dom.NodeList;
import assays.com.Util;

public final class AssayUser extends AssayPersistent {
  private String loginName;
  private Integer adminRights;
  private Integer startWithWelcome;
  private String password;
  private String idPrefix;
  private String ftpDirectory;
  private Integer ftpIsAllowed;
  private String userSettings;
  private String userSettingsFileExport;
  private UserGroup defaultUserGroup;
  private AssayProjectPartner projectPartner=null;
// -------------------------------------------
// -------------------------------------------
  private int viewLimit = 100;
  private boolean showFullId = false;
  private int administrationRights = 0;

  public boolean isPowerAdministrator() {
    return administrationRights >= 3000;
  }
  public boolean isWriteAdministrator() {
    return administrationRights >= 2000;
  }
  public boolean isAdministrator() {
    return administrationRights >= 1000;
  }
  public AssayProjectPartner getProjectPartner() {
	return projectPartner;
  }
  public UserGroup getDefaultUserGroup() {
    return defaultUserGroup;
  }
  public int getAdministrationRights() {
    return administrationRights;
  }
  public int getViewLimit() {
    return viewLimit;
  }
  public boolean isShowFullId() {
    return showFullId;
  }
  public void setViewLimit( int val) {
    viewLimit = val;
  }
  public void setShowFullId(boolean val) {
    showFullId = val;
  }
// -------------------------------------------
  private static final String Xsettings = "User_Settings";
  private static final String Xversion = "version";
  private static final String XUserConf = "UserConf";
  private static final String Xviewlimit = "view_limit";
  private static final String XshowFullId = "show_full_id";

  private String ftpUsername = null;
  private String ftpPassword = null;
  private String ftpServer = null;
  private String ftpRemotedirectory = null;
  public void importSettings( int defViewLimit, boolean defShowFullId){// throws Exception {
    viewLimit = defViewLimit; showFullId = defShowFullId;
    administrationRights = (adminRights!=null)?adminRights.intValue():0;

    if (userSettings!=null && !userSettings.equals(""))
    try {
      InputSource is = new InputSource( new StringReader(userSettings));
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = dbf.newDocumentBuilder();
      Document doc = db.parse(is);
      Element root = doc.getDocumentElement();
      root.normalize();
//      result = processRoot( root);
      String version = root.getAttribute(Xversion);
      if (version==null||!version.equals("1.17")){
//        errorlist.add("Invalid version");
        return;
      }
      NodeList nodeLst1 = root.getElementsByTagName(XUserConf);
      int num = nodeLst1.getLength();
      if ( num != 1){
//        errorlist.add("Invalid parameter groups");
        return;
      }
      Element userConf = (Element)nodeLst1.item(0);
      String attrval;
      attrval = userConf.getAttribute(Xviewlimit);
      viewLimit = Util.getIntegerValue(attrval);
      attrval = userConf.getAttribute(XshowFullId);
      showFullId = (attrval!=null && attrval.equals("1"));
/*
      ftpUsername = userConf.getAttribute(MetadataManager.getPropertyName("ftpusername"));
      ftpPassword = userConf.getAttribute(MetadataManager.getPropertyName("ftpuserpassword"));
      ftpServer = userConf.getAttribute(MetadataManager.getPropertyName("ftpservername"));
      ftpRemotedirectory = userConf.getAttribute(MetadataManager.getPropertyName("ftpdirectory"));
*/      
    }
    catch (Exception e) {
    }
  }
  public void importSettingsFileExport(){// throws Exception {
	if (userSettingsFileExport!=null && !userSettingsFileExport.equals(""))
    try {
// 	Util.writeLog("userSettingsFileExport", userSettingsFileExport, "");
      InputSource is = new InputSource( new StringReader(userSettingsFileExport));
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = dbf.newDocumentBuilder();
      Document doc = db.parse(is);
      Element root = doc.getDocumentElement();
      root.normalize();
      NodeList nodeLst1 = root.getElementsByTagName("context-param");
      ftpUsername = null;
      ftpPassword = null;
      ftpServer = null;
      ftpRemotedirectory = null;
      for (int i = 0; i < nodeLst1.getLength(); i++){
    	Element contpar = (Element)nodeLst1.item(i);
    	NodeList nodeLst2 = contpar.getElementsByTagName("param-name");
    	if (nodeLst2.getLength()!=1) continue;
    	Element paramname = (Element)nodeLst2.item(0);
    	NodeList nodeLst3 = paramname.getChildNodes();
    	if (nodeLst3.getLength()!=1) continue;
    	String thisname = nodeLst3.item(0).getNodeValue();
    	nodeLst2 = contpar.getElementsByTagName("param-value");
    	if (nodeLst2.getLength()!=1) continue;
    	Element paramvalue = (Element)nodeLst2.item(0);
    	nodeLst3 = paramvalue.getChildNodes();
    	if (nodeLst3.getLength()!=1) continue;
    	String thisvalue = nodeLst3.item(0).getNodeValue();
    	if (thisname.equals(MetadataManager.getPropertyName("ftpusername")))
    		ftpUsername = thisvalue; else
    	if (thisname.equals(MetadataManager.getPropertyName("ftpuserpassword")))
    		ftpPassword = thisvalue; else
    	if (thisname.equals(MetadataManager.getPropertyName("ftpservername")))
    		ftpServer = thisvalue; else
    	if (thisname.equals(MetadataManager.getPropertyName("ftpdirectory")))
    		ftpRemotedirectory = thisvalue;
      }
    }
    catch (Exception e) {
    }
  }

  private Stack<String> tagStack;
  private boolean isStarted = false;
  private StringBuffer jspOut;
  private String prevName;

  private void startTag( String name) {
    if (isStarted){
      jspOut.append('>');
      tagStack.push(prevName);
    }
    jspOut.append("\r\n");
    jspOut.append('<');
    jspOut.append(name);
    prevName = name;
    isStarted = true;
  }
  private void endTag() {
    if (isStarted){
      jspOut.append("/>");
      isStarted = false;
    } else {
      prevName = (String)tagStack.pop();
      jspOut.append("</");
      jspOut.append(prevName);
      jspOut.append('>');
    }
  }
  private void attribute(String name, String value) {
    jspOut.append(' ');
    jspOut.append(name);
    jspOut.append("=\"");
    jspOut.append(value);
    jspOut.append('\"');
  }
  private void valueOut( String name, String value){
    startTag("context-param");
	startTag( "param-name");  
    jspOut.append(">");
    isStarted = false;
    jspOut.append(name);
    tagStack.push(prevName);
    endTag();
	startTag( "param-value");  
    jspOut.append(">");
    isStarted = false;
    jspOut.append(value);
    tagStack.push(prevName);
    endTag();
    endTag();
  }

  public void exportSettings() {
    jspOut = new StringBuffer();
    tagStack = new Stack<String>();
    isStarted = false;
    startTag(Xsettings);
    attribute(Xversion,"1.17");
    startTag(XUserConf);
    attribute(Xviewlimit, String.valueOf( viewLimit));
    attribute(XshowFullId, isShowFullId() ? "1" : "0");
    endTag(); //User conf
    endTag();//User settings
    userSettings = jspOut.toString();
  }
  public void exportSettingsFileExport() {
    jspOut = new StringBuffer();
    tagStack = new Stack<String>();
    isStarted = false;
    jspOut.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    startTag("P");
    if (ftpUsername!=null) valueOut(MetadataManager.getPropertyName("ftpusername"), ftpUsername);
    if (ftpPassword!=null) valueOut(MetadataManager.getPropertyName("ftpuserpassword"), ftpPassword);
    if (ftpServer!=null) valueOut(MetadataManager.getPropertyName("ftpservername"), ftpServer);
    if (ftpRemotedirectory!=null) valueOut(MetadataManager.getPropertyName("ftpdirectory"), ftpRemotedirectory);
    endTag();//User settings
    userSettingsFileExport = jspOut.toString();
  }
  public String makeUserSettingsFileExport(String username, String password, String server, String directory) {
    jspOut = new StringBuffer();
    tagStack = new Stack<String>();
    isStarted = false;
    jspOut.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
    startTag("P");
    if (username!=null) valueOut(MetadataManager.getPropertyName("ftpusername"), username);
    if (password!=null) valueOut(MetadataManager.getPropertyName("ftpuserpassword"), password);
    if (server!=null) valueOut(MetadataManager.getPropertyName("ftpservername"), server);
    if (directory!=null) valueOut(MetadataManager.getPropertyName("ftpdirectory"), directory);
    endTag();//User settings
    return jspOut.toString();
  }

// -------------------------------------------
  public String getIdAsTitle(){
    return "title=\"Id="+this.getId().toString()+"\"";
  }

  public boolean isEqual( AssayUser otherUser){
    return otherUser!=null && getId().equals(otherUser.getId());
  }
  public Integer getFtpIsAllowed() {
    return ftpIsAllowed;
  }
  public boolean isFtpAllowed() {
    return ftpIsAllowed!=null && ftpIsAllowed.byteValue() > 0;
  }
  public Integer getStartWithWelcome() {
    return startWithWelcome;
  }
  public boolean isWithWelcomeStart() {
    return startWithWelcome!=null && startWithWelcome.byteValue() > 0;
  }

  public String getFtpDirectory() {
    return ftpDirectory;
  }
  public String getIdPrefix() {
    return idPrefix;
  }
  public String getLoginName() {
    return loginName;
  }
  public Integer getAdminRights() {
    return adminRights;
  }

  public String getPassword() {
    return password;
  }

  public String getUserSettings() {
    return userSettings;
  }
  public String getUserSettingsFileExport() {
    return userSettingsFileExport;
  }
  public String getFtpUserName() {
    return (ftpUsername==null)?"":ftpUsername;
  }
  public String getFtpPassWord() {
    return (ftpPassword==null)?"":ftpPassword;
  }
  public String getFtpServer() {
    return (ftpServer==null)?"":ftpServer;
  }
  public String getFtpRemoteDirectory() {
	    return (ftpRemotedirectory==null)?"":ftpRemotedirectory;
	  }
  public void setFtpUserName(String val) {
	ftpUsername = val;
  }
  public void setFtpPassWord(String val) {
	ftpPassword = val;
  }
  public void setFtpServer(String val) {
    ftpServer = val;
  }
  public void setFtpRemoteDirectory(String val) {
    ftpRemotedirectory = val;
  }
// -------------------------------------------
// -------------------------------------------
  public void setFtpIsAllowed(Integer val) {
    ftpIsAllowed = val;
  }
  public void setStartWithWelcome(Integer val) {
    startWithWelcome = val;
  }
  public void setFtpDirectory(String val) {
    ftpDirectory = val;
  }
  public void setIdPrefix(String val) {
    idPrefix = val;
  }
  public void setLoginName(String string) {
    loginName = string;
  }
  public void setProjectPartner(AssayProjectPartner val) {
    projectPartner = val;
  }
  public void setDefaultUserGroup(UserGroup val) {
    defaultUserGroup = val;
  }
  public void setAdminRights(Integer val) {
    adminRights = val;
  }
  public void setPassword(String string) {
    password = string;
  }

  public String toString() {
    return this.getId() + "  " + getVisibleName() + " : " + loginName;
  }

  public void setUserSettings( String val) {
    userSettings = val;
  }
  public void setUserSettingsFileExport( String val) {
    userSettingsFileExport = val;
  }
  private static final String[] accessVariants = {
      "non-accessible",
      "view only",
      "limited editing",
      "full editing"
  };
  private static final int[] accessValues = {
      0, 1000, 2000, 3000
  };
  public static String getAdminRightSelectionNew(){
    StringBuffer xyz = new StringBuffer();
    for (int i = 0; i < accessVariants.length; i++){
        xyz.append("<option ");
        xyz.append("value=\"")
            .append( accessValues[i])
            .append("\">")
            .append(accessVariants[i]);
      }
    return xyz.toString();
  }

  public String getAdminRightSelection(){
    StringBuffer xyz = new StringBuffer();
    int j = 1; int v = (adminRights!=null)?adminRights.intValue():0;
    while ( j < accessValues.length && v >= accessValues[j]) j++;
    j--;
    for (int i = 0; i < accessVariants.length; i++){
        xyz.append("<option ");
        if ( i==j) xyz.append("selected ");
        xyz.append("value=\"")
            .append( accessValues[i])
            .append("\">")
            .append(accessVariants[i]);
      }
    return xyz.toString();
  }
  public String getAdminRightText(){
    int j = 1; int v = (adminRights!=null)?adminRights.intValue():0;
    while ( j < accessValues.length && v >= accessValues[j]) j++;
    return accessVariants[--j];
  }
}
