package assays.hibernate;

public final class TempMetaData {
    private String suffix;     //used by upload
    private String shortName;  //used by view
    private String longName;
    private String uploadName;
    private String columnName; //used by view sort, also key in HashMap
    private int tempColumnNumber;//used by batch upload
    private int viewColumnNumber;
    private boolean sortable;
    private int valueNumber = 0;
    private int baseType = 0;
    private String groupName = null; //group shortname used by view
    private String groupLongName = null; //group longname
    private int attribute1 = 0;
    private int attribute2 = 0;
    private Long vocabularyId = null;
    private boolean required = false;
    private boolean showInFilter = false;
    private int level;
    public TempMetaData(AbstractParameter metadata, int lev, boolean isParameter) {
      shortName = new String(metadata.getVisibleName());
      longName = new String(metadata.getLongName());
      uploadName = metadata.getUploadName();
      if (uploadName==null||uploadName.equals("")) uploadName = longName;
      showInFilter = metadata.isShowInFilter();
      String cName = (lev==2)?metadata.getAssayColumnName():
                     (lev==1)?metadata.getStgrColumnName():
                     metadata.getViewColumnName();
      if (isParameter && cName != null && cName.length() == 4)
        columnName = "f" + cName.substring( 1);
      else
        columnName = cName;
      level = lev;
      if (level==2){
        tempColumnNumber = metadata.getAssayViewColumnNumber().intValue();
        sortable = metadata.isAssaySortable();
      }
      else if (level==1){
        tempColumnNumber = metadata.getStgrViewColumnNumber().intValue();
        sortable = metadata.isStgrSortable();
      }
      else {
        tempColumnNumber = metadata.getViewViewColumnNumber().intValue();
        sortable = metadata.isViewSortable();
      }
    }

    public boolean isBottomLevel(){
      return level == 2;
    }
    public boolean isMiddleLevel(){
      return level == 1;
    }
    public boolean isTopLevel(){
      return level == 0;
    }
    public String getSuffix(){
      return suffix;
    }
    public String getShortName(){
      return shortName;
    }
    public String getLongName(){
      return longName;
    }
    public String getUploadName(){
      return uploadName;
    }
    public String getColumnName(){
      return columnName;
    }
    public int getTempColumnNumber(){
      return tempColumnNumber;
    }
    public int getViewColumnNumber(){
      return viewColumnNumber;
    }
    public boolean getSortable(){
      return sortable;
    }
    public int getValueNumber(){
      return valueNumber;
    }
    public int getBaseType(){
      return baseType;
    }
    public String getGroupName(){
      return (groupName==null)?"":groupName;
    }
    public String getGroupLongName(){
      return (groupLongName==null)?"":groupLongName;
    }
    public int getAttribute1(){
      return attribute1;
    }
    public int getAttribute2(){
      return attribute2;
    }
    public Long getVocabularyId(){
      return vocabularyId;
    }
    public boolean isRequired() {
      return required;
    }
    public boolean isShowInFilter() {
      return showInFilter;
    }
//--------------------------------------------
    public void setUploadName(String val){
      uploadName = val;
    }
    public void setShortName(String val){
      shortName = val;
    }
    public void setLongName(String val){
      longName = val;
    }
    public void setColumnName(String val){
      columnName = val;
    }
    public void setViewColumnNumber(int val){
      viewColumnNumber = val;
    }
    public void setValueNumber(int val){
      valueNumber = val;
    }
    public void setBaseType(int val){
      baseType = val;
    }
    public void setSuffix(String val){
      suffix = val;
    }
    public void setGroupName(String val){
//      groupName = new String(val);
      groupName = val;
    }
    public void setGroupLongName(String val){
      groupLongName = val;
    }
    public void setAttribute1(int val){
      attribute1 = val;
    }
    public void setAttribute2(int val){
      attribute2 = val;
    }
    public void setVocabularyId(Long val){
      vocabularyId = val;
    }
    public void setTempColumnNumber(int val){
      tempColumnNumber = val;
    }
    public void setRequired(boolean val) {
      required = val;
    }
  }
